@@ -1,7 +1,7 @@
 /*
  * common.c
  *
- * Copyright (C) 2012 - 2015 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2016 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with Profanity.  If not, see <http://www.gnu.org/licenses/>.
+ * along with Profanity.  If not, see <https://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give permission to
  * link the code of portions of this program with the OpenSSL library under
@@ -33,10 +33,12 @@
  */
 #include "config.h"
 
-#include <assert.h>
 #include <errno.h>
+#include <sys/select.h>
+#include <assert.h>
 #include <stdlib.h>
 #include <stdint.h>
+#include <stdio.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -45,11 +47,15 @@
 #include <curl/easy.h>
 #include <glib.h>
 
-#include "tools/p_sha1.h"
+#ifdef HAVE_NCURSESW_NCURSES_H
+#include <ncursesw/ncurses.h>
+#elif HAVE_NCURSES_H
+#include <ncurses.h>
+#endif
 
 #include "log.h"
 #include "common.h"
-
+#include "tools/p_sha1.h"
 
 struct curl_data_t
 {
@@ -62,7 +68,7 @@ static unsigned long unique_id = 0;
 static size_t _data_callback(void *ptr, size_t size, size_t nmemb, void *data);
 
 // taken from glib 2.30.3
-gchar *
+gchar*
 p_utf8_substring(const gchar *str, glong start_pos, glong end_pos)
 {
     gchar *start, *end, *out;
@@ -102,7 +108,7 @@ p_hash_table_add(GHashTable *hash_table, gpointer key)
 }
 
 gboolean
-p_hash_table_contains(GHashTable  *hash_table, gconstpointer key)
+p_hash_table_contains(GHashTable *hash_table, gconstpointer key)
 {
     // doesn't handle when key exists, but value == NULL
     gpointer found = g_hash_table_lookup(hash_table, key);
@@ -148,7 +154,32 @@ mkdir_recursive(const char *dir)
     return result;
 }
 
-char *
+gboolean
+copy_file(const char *const sourcepath, const char *const targetpath)
+{
+    int ch;
+    FILE *source = fopen(sourcepath, "rb");
+    if (source == NULL) {
+        return FALSE;
+    }
+
+    FILE *target = fopen(targetpath, "wb");
+    if (target == NULL) {
+        fclose(source);
+        return FALSE;
+    }
+
+    while((ch = fgetc(source)) != EOF) {
+        fputc(ch, target);
+    }
+
+    fclose(source);
+    fclose(target);
+
+    return TRUE;
+}
+
+char*
 str_replace(const char *string, const char *substr,
     const char *replacement)
 {
@@ -192,18 +223,6 @@ str_replace(const char *string, const char *substr,
     return newstr;
 }
 
-gboolean
-str_contains_str(const char *  const searchstr, const char * const substr)
-{
-    if (!searchstr) {
-        return FALSE;
-    }
-    if (!substr) {
-        return FALSE;
-    }
-    return g_strrstr(searchstr, substr) != NULL;
-}
-
 int
 str_contains(const char str[], int size, char ch)
 {
@@ -244,7 +263,7 @@ strtoi_range(char *str, int *saveptr, int min, int max, char **err_msg)
 }
 
 int
-utf8_display_len(const char * const str)
+utf8_display_len(const char *const str)
 {
     if (!str) {
         return 0;
@@ -265,8 +284,8 @@ utf8_display_len(const char * const str)
     return len;
 }
 
-char *
-prof_getline(FILE *stream)
+char*
+file_getline(FILE *stream)
 {
     char *buf;
     char *result;
@@ -309,8 +328,8 @@ prof_getline(FILE *stream)
     return s;
 }
 
-char *
-release_get_latest()
+char*
+release_get_latest(void)
 {
     char *url = "http://www.profanity.im/profanity_version.txt";
 
@@ -361,114 +380,7 @@ release_is_new(char *found_version)
     }
 }
 
-gboolean
-valid_resource_presence_string(const char * const str)
-{
-    assert(str != NULL);
-    if ((strcmp(str, "online") == 0) || (strcmp(str, "chat") == 0) ||
-            (strcmp(str, "away") == 0) || (strcmp(str, "xa") == 0) ||
-            (strcmp(str, "dnd") == 0)) {
-        return TRUE;
-    } else {
-        return FALSE;
-    }
-}
-
-const char *
-string_from_resource_presence(resource_presence_t presence)
-{
-    switch(presence)
-    {
-        case RESOURCE_CHAT:
-            return "chat";
-        case RESOURCE_AWAY:
-            return "away";
-        case RESOURCE_XA:
-            return "xa";
-        case RESOURCE_DND:
-            return "dnd";
-        default:
-            return "online";
-    }
-}
-
-resource_presence_t
-resource_presence_from_string(const char * const str)
-{
-    if (str == NULL) {
-        return RESOURCE_ONLINE;
-    } else if (strcmp(str, "online") == 0) {
-        return RESOURCE_ONLINE;
-    } else if (strcmp(str, "chat") == 0) {
-        return RESOURCE_CHAT;
-    } else if (strcmp(str, "away") == 0) {
-        return RESOURCE_AWAY;
-    } else if (strcmp(str, "xa") == 0) {
-        return RESOURCE_XA;
-    } else if (strcmp(str, "dnd") == 0) {
-        return RESOURCE_DND;
-    } else {
-        return RESOURCE_ONLINE;
-    }
-}
-
-contact_presence_t
-contact_presence_from_resource_presence(resource_presence_t resource_presence)
-{
-    switch(resource_presence)
-    {
-        case RESOURCE_CHAT:
-            return CONTACT_CHAT;
-        case RESOURCE_AWAY:
-            return CONTACT_AWAY;
-        case RESOURCE_XA:
-            return CONTACT_XA;
-        case RESOURCE_DND:
-            return CONTACT_DND;
-        default:
-            return CONTACT_ONLINE;
-    }
-}
-
-gchar *
-xdg_get_config_home(void)
-{
-    gchar *xdg_config_home = getenv("XDG_CONFIG_HOME");
-    if (xdg_config_home)
-        g_strstrip(xdg_config_home);
-
-    if (xdg_config_home && (strcmp(xdg_config_home, "") != 0)) {
-        return strdup(xdg_config_home);
-    } else {
-        GString *default_path = g_string_new(getenv("HOME"));
-        g_string_append(default_path, "/.config");
-        gchar *result = strdup(default_path->str);
-        g_string_free(default_path, TRUE);
-
-        return result;
-    }
-}
-
-gchar *
-xdg_get_data_home(void)
-{
-    gchar *xdg_data_home = getenv("XDG_DATA_HOME");
-    if (xdg_data_home)
-        g_strstrip(xdg_data_home);
-
-    if (xdg_data_home && (strcmp(xdg_data_home, "") != 0)) {
-        return strdup(xdg_data_home);
-    } else {
-        GString *default_path = g_string_new(getenv("HOME"));
-        g_string_append(default_path, "/.local/share");
-        gchar *result = strdup(default_path->str);
-        g_string_free(default_path, TRUE);
-
-        return result;
-    }
-}
-
-char *
+char*
 create_unique_id(char *prefix)
 {
     char *result = NULL;
@@ -492,7 +404,7 @@ reset_unique_id(void)
     unique_id = 0;
 }
 
-char *
+char*
 p_sha1_hash(char *str)
 {
     P_SHA1_CTX ctx;
@@ -508,79 +420,6 @@ p_sha1_hash(char *str)
     return g_base64_encode(digest, sizeof(digest));
 }
 
-int
-cmp_win_num(gconstpointer a, gconstpointer b)
-{
-    int real_a = GPOINTER_TO_INT(a);
-    int real_b = GPOINTER_TO_INT(b);
-
-    if (real_a == 0) {
-        real_a = 10;
-    }
-
-    if (real_b == 0) {
-        real_b = 10;
-    }
-
-    if (real_a < real_b) {
-        return -1;
-    } else if (real_a == real_b) {
-        return 0;
-    } else {
-        return 1;
-    }
-}
-
-int
-get_next_available_win_num(GList *used)
-{
-    // only console used
-    if (g_list_length(used) == 1) {
-        return 2;
-    } else {
-        GList *sorted = NULL;
-        GList *curr = used;
-        while (curr) {
-            sorted = g_list_insert_sorted(sorted, curr->data, cmp_win_num);
-            curr = g_list_next(curr);
-        }
-
-        int result = 0;
-        int last_num = 1;
-        curr = sorted;
-        // skip console
-        curr = g_list_next(curr);
-        while (curr) {
-            int curr_num = GPOINTER_TO_INT(curr->data);
-
-            if (((last_num != 9) && ((last_num + 1) != curr_num)) ||
-                    ((last_num == 9) && (curr_num != 0))) {
-                result = last_num + 1;
-                if (result == 10) {
-                    result = 0;
-                }
-                g_list_free(sorted);
-                return (result);
-
-            } else {
-                last_num = curr_num;
-                if (last_num == 0) {
-                    last_num = 10;
-                }
-            }
-            curr = g_list_next(curr);
-        }
-        result = last_num + 1;
-        if (result == 10) {
-            result = 0;
-        }
-
-        g_list_free(sorted);
-        return result;
-    }
-}
-
-
 static size_t
 _data_callback(void *ptr, size_t size, size_t nmemb, void *data)
 {
@@ -623,8 +462,8 @@ get_file_or_linked(char *loc, char *basedir)
     return true_loc;
 }
 
-char *
-strip_arg_quotes(const char * const input)
+char*
+strip_arg_quotes(const char *const input)
 {
     char *unquoted = strdup(input);
 
@@ -644,3 +483,50 @@ strip_arg_quotes(const char * const input)
 
     return unquoted;
 }
+
+gboolean
+is_notify_enabled(void)
+{
+    gboolean notify_enabled = FALSE;
+
+#ifdef HAVE_OSXNOTIFY
+    notify_enabled = TRUE;
+#endif
+#ifdef HAVE_LIBNOTIFY
+    notify_enabled = TRUE;
+#endif
+#ifdef PLATFORM_CYGWIN
+    notify_enabled = TRUE;
+#endif
+
+    return notify_enabled;
+}
+
+GSList*
+prof_occurrences(const char *const needle, const char *const haystack, int offset, gboolean whole_word, GSList **result)
+{
+    if (needle == NULL || haystack == NULL) {
+        return *result;
+    }
+
+    if (g_str_has_prefix(&haystack[offset], needle)) {
+        if (whole_word) {
+            char *prev = g_utf8_prev_char(&haystack[offset]);
+            char *next = g_utf8_next_char(&haystack[offset] + strlen(needle) - 1);
+            gunichar prevu = g_utf8_get_char(prev);
+            gunichar nextu = g_utf8_get_char(next);
+            if (!g_unichar_isalnum(prevu) && !g_unichar_isalnum(nextu)) {
+                *result = g_slist_append(*result, GINT_TO_POINTER(offset));
+            }
+        } else {
+            *result = g_slist_append(*result, GINT_TO_POINTER(offset));
+        }
+    }
+
+    if (haystack[offset+1] != '\0') {
+        *result = prof_occurrences(needle, haystack, offset+1, whole_word, result);
+    }
+
+    return *result;
+}
+
