@@ -1,7 +1,7 @@
 /*
  * main.c
  *
- * Copyright (C) 2012 - 2015 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2016 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with Profanity.  If not, see <http://www.gnu.org/licenses/>.
+ * along with Profanity.  If not, see <https://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give permission to
  * link the code of portions of this program with the OpenSSL library under
@@ -31,18 +31,20 @@
  * source files in the program, then also delete it here.
  *
  */
+
+#include "config.h"
+
 #include <string.h>
 #include <glib.h>
 
-#include "config.h"
 #ifdef HAVE_GIT_VERSION
 #include "gitversion.h"
 #endif
 
 #include "profanity.h"
-#include "command/command.h"
+#include "common.h"
+#include "command/cmd_defs.h"
 
-static gboolean disable_tls = FALSE;
 static gboolean version = FALSE;
 static char *log = "INFO";
 static char *account_name = NULL;
@@ -58,7 +60,6 @@ main(int argc, char **argv)
     static GOptionEntry entries[] =
     {
         { "version", 'v', 0, G_OPTION_ARG_NONE, &version, "Show version information", NULL },
-        { "disable-tls", 'd', 0, G_OPTION_ARG_NONE, &disable_tls, "Disable TLS", NULL },
         { "account", 'a', 0, G_OPTION_ARG_STRING, &account_name, "Auto connect to an account on startup" },
         { "log",'l', 0, G_OPTION_ARG_STRING, &log, "Set logging levels, DEBUG, INFO (default), WARN, ERROR", "LEVEL" },
         { NULL }
@@ -89,8 +90,8 @@ main(int argc, char **argv)
             g_print("Profanity, version %s\n", PACKAGE_VERSION);
         }
 
-        g_print("Copyright (C) 2012 - 2015 James Booth <%s>.\n", PACKAGE_BUGREPORT);
-        g_print("License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n");
+        g_print("Copyright (C) 2012 - 2016 James Booth <%s>.\n", PACKAGE_BUGREPORT);
+        g_print("License GPLv3+: GNU GPL version 3 or later <https://www.gnu.org/licenses/gpl.html>\n");
         g_print("\n");
         g_print("This is free software; you are free to change and redistribute it.\n");
         g_print("There is NO WARRANTY, to the extent permitted by law.\n");
@@ -98,18 +99,14 @@ main(int argc, char **argv)
 
         g_print("Build information:\n");
 
-        gboolean notify_enabled = FALSE;
+#ifdef HAVE_LIBMESODE
+        g_print("XMPP library: libmesode\n");
+#endif
+#ifdef HAVE_LIBSTROPHE
+        g_print("XMPP library: libstrophe\n");
+#endif
 
-#ifdef HAVE_OSXNOTIFY
-        notify_enabled = TRUE;
-#endif
-#ifdef HAVE_LIBNOTIFY
-        notify_enabled = TRUE;
-#endif
-#ifdef PLATFORM_CYGWIN
-        notify_enabled = TRUE;
-#endif
-        if (notify_enabled) {
+        if (is_notify_enabled()) {
             g_print("Desktop notification support: Enabled\n");
         } else {
             g_print("Desktop notification support: Disabled\n");
@@ -127,10 +124,28 @@ main(int argc, char **argv)
         g_print("PGP support: Disabled\n");
 #endif
 
+#ifdef HAVE_C
+        g_print("C plugins: Enabled\n");
+#else
+        g_print("C plugins: Disabled\n");
+#endif
+
+#ifdef HAVE_PYTHON
+        g_print("Python plugins: Enabled\n");
+#else
+        g_print("Python plugins: Disabled\n");
+#endif
+
+#ifdef HAVE_GTK
+        g_print("GTK icons: Enabled\n");
+#else
+        g_print("GTK icons: Disabled\n");
+#endif
+
         return 0;
     }
 
-    prof_run(disable_tls, log, account_name);
+    prof_run(log, account_name);
 
     return 0;
 }
