@@ -1,7 +1,7 @@
 /*
  * otr.h
  *
- * Copyright (C) 2012 - 2015 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2016 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with Profanity.  If not, see <http://www.gnu.org/licenses/>.
+ * along with Profanity.  If not, see <https://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give permission to
  * link the code of portions of this program with the OpenSSL library under
@@ -32,14 +32,14 @@
  *
  */
 
-#ifndef OTR_H
-#define OTR_H
+#ifndef OTR_OTR_H
+#define OTR_OTR_H
 
 #include <libotr/proto.h>
 #include <libotr/message.h>
 
 #include "config/accounts.h"
-#include "ui/ui.h"
+#include "ui/win_types.h"
 
 typedef enum {
     PROF_OTRPOLICY_MANUAL,
@@ -47,6 +47,19 @@ typedef enum {
     PROF_OTRPOLICY_ALWAYS
 } prof_otrpolicy_t;
 
+typedef enum {
+    PROF_OTR_SMP_INIT,
+    PROF_OTR_SMP_INIT_Q,
+    PROF_OTR_SMP_SENDER_FAIL,
+    PROF_OTR_SMP_RECEIVER_FAIL,
+    PROF_OTR_SMP_ABORT,
+    PROF_OTR_SMP_SUCCESS,
+    PROF_OTR_SMP_SUCCESS_Q,
+    PROF_OTR_SMP_FAIL_Q,
+    PROF_OTR_SMP_AUTH,
+    PROF_OTR_SMP_AUTH_WAIT
+} prof_otr_smp_event_t;
+
 OtrlUserState otr_userstate(void);
 OtrlMessageAppOps* otr_messageops(void);
 GHashTable* otr_smpinitators(void);
@@ -58,35 +71,35 @@ char* otr_start_query(void);
 void otr_poll(void);
 void otr_on_connect(ProfAccount *account);
 
-char* otr_on_message_recv(const char * const barejid, const char * const resource, const char * const message, gboolean *decrypted);
-gboolean otr_on_message_send(ProfChatWin *chatwin, const char * const message);
+char* otr_on_message_recv(const char *const barejid, const char *const resource, const char *const message, gboolean *decrypted);
+gboolean otr_on_message_send(ProfChatWin *chatwin, const char *const message, gboolean request_receipt);
 
 void otr_keygen(ProfAccount *account);
 
-char* otr_tag_message(const char * const msg);
+char* otr_tag_message(const char *const msg);
 
 gboolean otr_key_loaded(void);
-gboolean otr_is_secure(const char * const recipient);
+gboolean otr_is_secure(const char *const recipient);
 
-gboolean otr_is_trusted(const char * const recipient);
-void otr_trust(const char * const recipient);
-void otr_untrust(const char * const recipient);
+gboolean otr_is_trusted(const char *const recipient);
+void otr_trust(const char *const recipient);
+void otr_untrust(const char *const recipient);
 
-void otr_smp_secret(const char * const recipient, const char *secret);
-void otr_smp_question(const char * const recipient, const char *question, const char *answer);
-void otr_smp_answer(const char * const recipient, const char *answer);
+void otr_smp_secret(const char *const recipient, const char *secret);
+void otr_smp_question(const char *const recipient, const char *question, const char *answer);
+void otr_smp_answer(const char *const recipient, const char *answer);
 
-void otr_end_session(const char * const recipient);
+void otr_end_session(const char *const recipient);
 
-char * otr_get_my_fingerprint(void);
-char * otr_get_their_fingerprint(const char * const recipient);
+char* otr_get_my_fingerprint(void);
+char* otr_get_their_fingerprint(const char *const recipient);
 
-char * otr_encrypt_message(const char * const to, const char * const message);
-char * otr_decrypt_message(const char * const from, const char * const message,
+char* otr_encrypt_message(const char *const to, const char *const message);
+char* otr_decrypt_message(const char *const from, const char *const message,
     gboolean *decrypted);
 
 void otr_free_message(char *message);
 
-prof_otrpolicy_t otr_get_policy(const char * const recipient);
+prof_otrpolicy_t otr_get_policy(const char *const recipient);
 
 #endif
