@@ -1,7 +1,7 @@
 /*
  * parser.h
  *
- * Copyright (C) 2012 - 2015 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2016 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with Profanity.  If not, see <http://www.gnu.org/licenses/>.
+ * along with Profanity.  If not, see <https://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give permission to
  * link the code of portions of this program with the OpenSSL library under
@@ -32,16 +32,16 @@
  *
  */
 
-#ifndef PARSER_H
-#define PARSER_H
+#ifndef TOOLS_PARSER_H
+#define TOOLS_PARSER_H
 
 #include <glib.h>
 
-gchar** parse_args(const char * const inp, int min, int max, gboolean *result);
-gchar** parse_args_with_freetext(const char * const inp, int min, int max, gboolean *result);
-int count_tokens(const char * const string);
-char* get_start(const char * const string, int tokens);
+gchar** parse_args(const char *const inp, int min, int max, gboolean *result);
+gchar** parse_args_with_freetext(const char *const inp, int min, int max, gboolean *result);
+int count_tokens(const char *const string);
+char* get_start(const char *const string, int tokens);
 GHashTable* parse_options(gchar **args, gchar **keys, gboolean *res);
 void options_destroy(GHashTable *options);
 
-#endif
\ No newline at end of file
+#endif
