@@ -1,7 +1,7 @@
 /*
  * buffer.c
  *
- * Copyright (C) 2012 - 2015 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2016 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with Profanity.  If not, see <http://www.gnu.org/licenses/>.
+ * along with Profanity.  If not, see <https://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give permission to
  * link the code of portions of this program with the OpenSSL library under
@@ -58,7 +58,7 @@ struct prof_buff_t {
 static void _free_entry(ProfBuffEntry *entry);
 
 ProfBuff
-buffer_create()
+buffer_create(void)
 {
     ProfBuff new_buff = malloc(sizeof(struct prof_buff_t));
     new_buff->entries = NULL;
@@ -76,12 +76,11 @@ buffer_free(ProfBuff buffer)
 {
     g_slist_free_full(buffer->entries, (GDestroyNotify)_free_entry);
     free(buffer);
-    buffer = NULL;
 }
 
 void
 buffer_push(ProfBuff buffer, const char show_char, int pad_indent, GDateTime *time,
-    int flags, theme_item_t theme_item, const char * const from, const char * const message, DeliveryReceipt *receipt)
+    int flags, theme_item_t theme_item, const char *const from, const char *const message, DeliveryReceipt *receipt)
 {
     ProfBuffEntry *e = malloc(sizeof(struct prof_buff_entry_t));
     e->show_char = show_char;
@@ -89,7 +88,7 @@ buffer_push(ProfBuff buffer, const char show_char, int pad_indent, GDateTime *ti
     e->flags = flags;
     e->theme_item = theme_item;
     e->time = g_date_time_ref(time);
-    e->from = strdup(from);
+    e->from = from ? strdup(from) : NULL;
     e->message = strdup(message);
     e->receipt = receipt;
 
@@ -102,7 +101,7 @@ buffer_push(ProfBuff buffer, const char show_char, int pad_indent, GDateTime *ti
 }
 
 gboolean
-buffer_mark_received(ProfBuff buffer, const char * const id)
+buffer_mark_received(ProfBuff buffer, const char *const id)
 {
     GSList *entries = buffer->entries;
     while (entries) {
@@ -126,6 +125,21 @@ buffer_yield_entry(ProfBuff buffer, int entry)
     return node->data;
 }
 
+ProfBuffEntry*
+buffer_yield_entry_by_id(ProfBuff buffer, const char *const id)
+{
+    GSList *entries = buffer->entries;
+    while (entries) {
+        ProfBuffEntry *entry = entries->data;
+        if (entry->receipt && g_strcmp0(entry->receipt->id, id) == 0) {
+            return entry;
+        }
+        entries = g_slist_next(entries);
+    }
+
+    return NULL;
+}
+
 static void
 _free_entry(ProfBuffEntry *entry)
 {
