@@ -1,7 +1,7 @@
 /*
  * connection.h
  *
- * Copyright (C) 2012 - 2015 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2016 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with Profanity.  If not, see <http://www.gnu.org/licenses/>.
+ * along with Profanity.  If not, see <https://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give permission to
  * link the code of portions of this program with the OpenSSL library under
@@ -35,15 +35,31 @@
 #ifndef XMPP_CONNECTION_H
 #define XMPP_CONNECTION_H
 
-#include <strophe.h>
+#include "xmpp/xmpp.h"
 
-#include "resource.h"
+void connection_init(void);
+void connection_shutdown(void);
+void connection_check_events(void);
 
-xmpp_conn_t *connection_get_conn(void);
-xmpp_ctx_t *connection_get_ctx(void);
+jabber_conn_status_t connection_connect(const char *const fulljid, const char *const passwd, const char *const altdomain, int port,
+    const char *const tls_policy);
+void connection_disconnect(void);
+void connection_set_disconnected(void);
+
+void connection_set_presence_msg(const char *const message);
+void connection_set_priority(const int priority);
 void connection_set_priority(int priority);
-void connection_set_presence_message(const char * const message);
+void connection_set_disco_items(GSList *items);
+
+xmpp_conn_t* connection_get_conn(void);
+xmpp_ctx_t* connection_get_ctx(void);
+char *connection_get_domain(void);
+char* connection_jid_for_feature(const char *const feature);
+GHashTable* connection_get_features(const char *const jid);
+
+void connection_clear_data(void);
+
 void connection_add_available_resource(Resource *resource);
-void connection_remove_available_resource(const char * const resource);
+void connection_remove_available_resource(const char *const resource);
 
 #endif
