@@ -1,7 +1,7 @@
 /*
  * form.c
  *
- * Copyright (C) 2012 - 2015 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2016 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with Profanity.  If not, see <http://www.gnu.org/licenses/>.
+ * along with Profanity.  If not, see <https://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give permission to
  * link the code of portions of this program with the OpenSSL library under
@@ -32,10 +32,19 @@
  *
  */
 
+#include "config.h"
+
 #include <string.h>
 #include <stdlib.h>
 
+#ifdef HAVE_LIBMESODE
+#include <mesode.h>
+#endif
+
+#ifdef HAVE_LIBSTROPHE
 #include <strophe.h>
+#endif
+
 #include <glib.h>
 
 #include "log.h"
@@ -46,19 +55,19 @@
 static gboolean
 _is_valid_form_element(xmpp_stanza_t *stanza)
 {
-    char *name = xmpp_stanza_get_name(stanza);
+    const char *name = xmpp_stanza_get_name(stanza);
     if (g_strcmp0(name, STANZA_NAME_X) != 0) {
         log_error("Error parsing form, root element not <x/>.");
         return FALSE;
     }
 
-    char *ns = xmpp_stanza_get_ns(stanza);
+    const char *ns = xmpp_stanza_get_ns(stanza);
     if (g_strcmp0(ns, STANZA_NS_DATA) != 0) {
         log_error("Error parsing form, namespace not %s.", STANZA_NS_DATA);
         return FALSE;
     }
 
-    char *type = xmpp_stanza_get_attribute(stanza, STANZA_ATTR_TYPE);
+    const char *type = xmpp_stanza_get_type(stanza);
     if ((g_strcmp0(type, "form") != 0) &&
             (g_strcmp0(type, "submit") != 0) &&
             (g_strcmp0(type, "cancel") != 0) &&
@@ -70,7 +79,7 @@ _is_valid_form_element(xmpp_stanza_t *stanza)
     return TRUE;
 }
 
-static DataForm *
+static DataForm*
 _form_new(void)
 {
     DataForm *form = malloc(sizeof(DataForm));
@@ -85,7 +94,7 @@ _form_new(void)
     return form;
 }
 
-static FormField *
+static FormField*
 _field_new(void)
 {
     FormField *field = malloc(sizeof(FormField));
@@ -100,8 +109,8 @@ _field_new(void)
     return field;
 }
 
-static char *
-_get_property(xmpp_stanza_t * const stanza, const char * const property)
+static char*
+_get_property(xmpp_stanza_t *const stanza, const char *const property)
 {
     char *result = NULL;
     xmpp_ctx_t *ctx = connection_get_ctx();
@@ -118,10 +127,10 @@ _get_property(xmpp_stanza_t * const stanza, const char * const property)
     return result;
 }
 
-static char *
-_get_attr(xmpp_stanza_t * const stanza, const char * const attr)
+static char*
+_get_attr(xmpp_stanza_t *const stanza, const char *const attr)
 {
-    char *result = xmpp_stanza_get_attribute(stanza, attr);
+    const char *result = xmpp_stanza_get_attribute(stanza, attr);
     if (result) {
         return strdup(result);
     } else {
@@ -130,7 +139,7 @@ _get_attr(xmpp_stanza_t * const stanza, const char * const attr)
 }
 
 static gboolean
-_is_required(xmpp_stanza_t * const stanza)
+_is_required(xmpp_stanza_t *const stanza)
 {
     xmpp_stanza_t *child = xmpp_stanza_get_child_by_name(stanza, "required");
     if (child) {
@@ -141,7 +150,7 @@ _is_required(xmpp_stanza_t * const stanza)
 }
 
 static form_field_type_t
-_get_field_type(const char * const type)
+_get_field_type(const char *const type)
 {
     if (g_strcmp0(type, "hidden") == 0) {
         return FIELD_HIDDEN;
@@ -176,8 +185,8 @@ _get_field_type(const char * const type)
     return FIELD_UNKNOWN;
 }
 
-DataForm *
-form_create(xmpp_stanza_t * const form_stanza)
+DataForm*
+form_create(xmpp_stanza_t *const form_stanza)
 {
     xmpp_ctx_t *ctx = connection_get_ctx();
 
@@ -199,7 +208,7 @@ form_create(xmpp_stanza_t * const form_stanza)
     // get fields
     xmpp_stanza_t *form_child = xmpp_stanza_get_children(form_stanza);
     while (form_child) {
-        char *child_name = xmpp_stanza_get_name(form_child);
+        const char *child_name = xmpp_stanza_get_name(form_child);
         if (g_strcmp0(child_name, "field") == 0) {
             xmpp_stanza_t *field_stanza = form_child;
 
@@ -273,7 +282,7 @@ form_create(xmpp_stanza_t * const form_stanza)
     return form;
 }
 
-xmpp_stanza_t *
+xmpp_stanza_t*
 form_create_submission(DataForm *form)
 {
     xmpp_ctx_t *ctx = connection_get_ctx();
@@ -400,7 +409,7 @@ _field_compare_by_var(FormField *a, FormField *b)
     return g_strcmp0(a->var, b->var);
 }
 
-GSList *
+GSList*
 form_get_non_form_type_fields_sorted(DataForm *form)
 {
     GSList *sorted = NULL;
@@ -416,7 +425,7 @@ form_get_non_form_type_fields_sorted(DataForm *form)
     return sorted;
 }
 
-GSList *
+GSList*
 form_get_field_values_sorted(FormField *field)
 {
     GSList *sorted = NULL;
@@ -432,7 +441,7 @@ form_get_field_values_sorted(FormField *field)
     return sorted;
 }
 
-char *
+char*
 form_get_form_type_field(DataForm *form)
 {
     GSList *curr = form->fields;
@@ -448,7 +457,7 @@ form_get_form_type_field(DataForm *form)
 }
 
 gboolean
-form_tag_exists(DataForm *form, const char * const tag)
+form_tag_exists(DataForm *form, const char *const tag)
 {
     GList *tags = g_hash_table_get_keys(form->tag_to_var);
     GList *curr = tags;
@@ -464,7 +473,7 @@ form_tag_exists(DataForm *form, const char * const tag)
 }
 
 form_field_type_t
-form_get_field_type(DataForm *form, const char * const tag)
+form_get_field_type(DataForm *form, const char *const tag)
 {
     char *var = g_hash_table_lookup(form->tag_to_var, tag);
     if (var) {
@@ -481,7 +490,7 @@ form_get_field_type(DataForm *form, const char * const tag)
 }
 
 void
-form_set_value(DataForm *form, const char * const tag, char *value)
+form_set_value(DataForm *form, const char *const tag, char *value)
 {
     char *var = g_hash_table_lookup(form->tag_to_var, tag);
     if (var) {
@@ -506,7 +515,7 @@ form_set_value(DataForm *form, const char * const tag, char *value)
 }
 
 void
-form_add_value(DataForm *form, const char * const tag, char *value)
+form_add_value(DataForm *form, const char *const tag, char *value)
 {
     char *var = g_hash_table_lookup(form->tag_to_var, tag);
     if (var) {
@@ -531,7 +540,7 @@ form_add_value(DataForm *form, const char * const tag, char *value)
 }
 
 gboolean
-form_add_unique_value(DataForm *form, const char * const tag, char *value)
+form_add_unique_value(DataForm *form, const char *const tag, char *value)
 {
     char *var = g_hash_table_lookup(form->tag_to_var, tag);
     if (var) {
@@ -562,7 +571,7 @@ form_add_unique_value(DataForm *form, const char * const tag, char *value)
 }
 
 gboolean
-form_remove_value(DataForm *form, const char * const tag, char *value)
+form_remove_value(DataForm *form, const char *const tag, char *value)
 {
     char *var = g_hash_table_lookup(form->tag_to_var, tag);
     if (var) {
@@ -592,7 +601,7 @@ form_remove_value(DataForm *form, const char * const tag, char *value)
 }
 
 gboolean
-form_remove_text_multi_value(DataForm *form, const char * const tag, int index)
+form_remove_text_multi_value(DataForm *form, const char *const tag, int index)
 {
     index--;
     char *var = g_hash_table_lookup(form->tag_to_var, tag);
@@ -624,7 +633,7 @@ form_remove_text_multi_value(DataForm *form, const char * const tag, int index)
 }
 
 int
-form_get_value_count(DataForm *form, const char * const tag)
+form_get_value_count(DataForm *form, const char *const tag)
 {
     char *var = g_hash_table_lookup(form->tag_to_var, tag);
     if (var) {
@@ -646,7 +655,7 @@ form_get_value_count(DataForm *form, const char * const tag)
 }
 
 gboolean
-form_field_contains_option(DataForm *form, const char * const tag, char *value)
+form_field_contains_option(DataForm *form, const char *const tag, char *value)
 {
     char *var = g_hash_table_lookup(form->tag_to_var, tag);
     if (var) {
@@ -670,8 +679,8 @@ form_field_contains_option(DataForm *form, const char * const tag, char *value)
     return FALSE;
 }
 
-FormField *
-form_get_field_by_tag(DataForm *form, const char * const tag)
+FormField*
+form_get_field_by_tag(DataForm *form, const char *const tag)
 {
     char *var = g_hash_table_lookup(form->tag_to_var, tag);
     if (var) {
@@ -688,7 +697,7 @@ form_get_field_by_tag(DataForm *form, const char * const tag)
 }
 
 Autocomplete
-form_get_value_ac(DataForm *form, const char * const tag)
+form_get_value_ac(DataForm *form, const char *const tag)
 {
     char *var = g_hash_table_lookup(form->tag_to_var, tag);
     if (var) {
@@ -714,4 +723,4 @@ form_reset_autocompleters(DataForm *form)
         autocomplete_reset(field->value_ac);
         curr_field = g_slist_next(curr_field);
     }
-}
\ No newline at end of file
+}
