@@ -1,7 +1,7 @@
 /*
  * form.h
  *
- * Copyright (C) 2012 - 2015 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2016 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with Profanity.  If not, see <http://www.gnu.org/licenses/>.
+ * along with Profanity.  If not, see <https://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give permission to
  * link the code of portions of this program with the OpenSSL library under
@@ -32,12 +32,23 @@
  *
  */
 
-#ifndef FORM_H
-#define FORM_H
+#ifndef XMPP_FORM_H
+#define XMPP_FORM_H
+
+#ifdef HAVE_LIBMESODE
+#include <mesode.h>
+#endif
+
+#ifdef HAVE_LIBSTROPHE
+#include <strophe.h>
+#endif
 
 #include "xmpp/xmpp.h"
 
-DataForm* form_create(xmpp_stanza_t * const stanza);
+DataForm* form_create(xmpp_stanza_t *const stanza);
 xmpp_stanza_t* form_create_submission(DataForm *form);
+char* form_get_form_type_field(DataForm *form);
+GSList* form_get_non_form_type_fields_sorted(DataForm *form);
+GSList* form_get_field_values_sorted(FormField *field);
 
 #endif
