@@ -1,7 +1,7 @@
 /*
  * message.c
  *
- * Copyright (C) 2012 - 2015 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2016 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with Profanity.  If not, see <http://www.gnu.org/licenses/>.
+ * along with Profanity.  If not, see <https://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give permission to
  * link the code of portions of this program with the OpenSSL library under
@@ -32,131 +32,156 @@
  *
  */
 
+#include "config.h"
+
 #include <stdlib.h>
 #include <string.h>
 
+#ifdef HAVE_LIBMESODE
+#include <mesode.h>
+#endif
+
+#ifdef HAVE_LIBSTROPHE
 #include <strophe.h>
+#endif
 
-#include "chat_session.h"
-#include "config/preferences.h"
-#include "log.h"
-#include "muc.h"
 #include "profanity.h"
-#include "ui/ui.h"
+#include "log.h"
+#include "config/preferences.h"
 #include "event/server_events.h"
-#include "xmpp/connection.h"
+#include "pgp/gpg.h"
+#include "plugins/plugins.h"
+#include "ui/ui.h"
+#include "xmpp/chat_session.h"
+#include "xmpp/muc.h"
+#include "xmpp/session.h"
 #include "xmpp/message.h"
 #include "xmpp/roster.h"
-#include "roster_list.h"
+#include "xmpp/roster_list.h"
 #include "xmpp/stanza.h"
+#include "xmpp/connection.h"
 #include "xmpp/xmpp.h"
-#include "pgp/gpg.h"
 
-#define HANDLE(ns, type, func) xmpp_handler_add(conn, func, ns, STANZA_NAME_MESSAGE, type, ctx)
+static int _message_handler(xmpp_conn_t *const conn, xmpp_stanza_t *const stanza, void *const userdata);
 
-static int _groupchat_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza, void * const userdata);
-static int _chat_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza, void * const userdata);
-static int _muc_user_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza, void * const userdata);
-static int _conference_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza, void * const userdata);
-static int _captcha_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza, void * const userdata);
-static int _message_error_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza, void * const userdata);
-static int _receipt_received_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza, void * const userdata);
+static void _handle_error(xmpp_stanza_t *const stanza);
+static void _handle_groupchat(xmpp_stanza_t *const stanza);
+static void _handel_muc_user(xmpp_stanza_t *const stanza);
+static void _handle_conference(xmpp_stanza_t *const stanza);
+static void _handle_captcha(xmpp_stanza_t *const stanza);
+static void _handle_receipt_received(xmpp_stanza_t *const stanza);
+static void _handle_chat(xmpp_stanza_t *const stanza);
+
+static void _send_message_stanza(xmpp_stanza_t *const stanza);
+
+static int
+_message_handler(xmpp_conn_t *const conn, xmpp_stanza_t *const stanza, void *const userdata)
+{
+    log_debug("Message stanza handler fired");
+
+    char *text;
+    size_t text_size;
+    xmpp_stanza_to_text(stanza, &text, &text_size);
+    gboolean cont = plugins_on_message_stanza_receive(text);
+    xmpp_free(connection_get_ctx(), text);
+    if (!cont) {
+        return 1;
+    }
+
+    const char *type = xmpp_stanza_get_type(stanza);
+
+    if (g_strcmp0(type, STANZA_TYPE_ERROR) == 0) {
+        _handle_error(stanza);
+    }
+
+    if (g_strcmp0(type, STANZA_TYPE_GROUPCHAT) == 0) {
+        _handle_groupchat(stanza);
+    }
+
+    xmpp_stanza_t *mucuser = xmpp_stanza_get_child_by_ns(stanza, STANZA_NS_MUC_USER);
+    if (mucuser) {
+        _handel_muc_user(stanza);
+    }
+
+    xmpp_stanza_t *conference = xmpp_stanza_get_child_by_ns(stanza, STANZA_NS_CONFERENCE);
+    if (conference) {
+        _handle_conference(stanza);
+    }
+
+    xmpp_stanza_t *captcha = xmpp_stanza_get_child_by_ns(stanza, STANZA_NS_CAPTCHA);
+    if (captcha) {
+        _handle_captcha(stanza);
+    }
+
+    xmpp_stanza_t *receipts = xmpp_stanza_get_child_by_ns(stanza, STANZA_NS_RECEIPTS);
+    if (receipts) {
+        _handle_receipt_received(stanza);
+    }
+
+    _handle_chat(stanza);
+
+    return 1;
+}
 
 void
-message_add_handlers(void)
+message_handlers_init(void)
 {
     xmpp_conn_t * const conn = connection_get_conn();
     xmpp_ctx_t * const ctx = connection_get_ctx();
-
-    HANDLE(NULL,                 STANZA_TYPE_ERROR,      _message_error_handler);
-    HANDLE(NULL,                 STANZA_TYPE_GROUPCHAT,  _groupchat_handler);
-    HANDLE(NULL,                 NULL,                   _chat_handler);
-    HANDLE(STANZA_NS_MUC_USER,   NULL,                   _muc_user_handler);
-    HANDLE(STANZA_NS_CONFERENCE, NULL,                   _conference_handler);
-    HANDLE(STANZA_NS_CAPTCHA,    NULL,                   _captcha_handler);
-    HANDLE(STANZA_NS_RECEIPTS,   NULL,                   _receipt_received_handler);
+    xmpp_handler_add(conn, _message_handler, NULL, STANZA_NAME_MESSAGE, NULL, ctx);
 }
 
-static char*
-_session_jid(const char * const barejid)
+char*
+message_send_chat(const char *const barejid, const char *const msg, const char *const oob_url,
+    gboolean request_receipt)
 {
-    ChatSession *session = chat_session_get(barejid);
-    char *jid = NULL;
-    if (session) {
-        Jid *jidp = jid_create_from_bare_and_resource(session->barejid, session->resource);
-        jid = strdup(jidp->fulljid);
-        jid_destroy(jidp);
-    } else {
-        jid = strdup(barejid);
-    }
-
-    return jid;
-}
-
-static char*
-_session_state(const char * const barejid)
-{
-    ChatSession *session = chat_session_get(barejid);
-    char *state = NULL;
-    if (session) {
-        if (prefs_get_boolean(PREF_STATES) && session->send_states) {
-            state = STANZA_NAME_ACTIVE;
-        }
-    } else {
-        if (prefs_get_boolean(PREF_STATES)) {
-            state = STANZA_NAME_ACTIVE;
-        }
-    }
-
-    return state;
-}
-
-char *
-message_send_chat(const char * const barejid, const char * const msg)
-{
-    xmpp_conn_t * const conn = connection_get_conn();
     xmpp_ctx_t * const ctx = connection_get_ctx();
 
-    char *state = _session_state(barejid);
-    char *jid = _session_jid(barejid);
+    char *state = chat_session_get_state(barejid);
+    char *jid = chat_session_get_jid(barejid);
     char *id = create_unique_id("msg");
 
-    xmpp_stanza_t *message = stanza_create_message(ctx, id, jid, STANZA_TYPE_CHAT, msg);
+    xmpp_stanza_t *message = xmpp_message_new(ctx, STANZA_TYPE_CHAT, jid, id);
+    xmpp_message_set_body(message, msg);
     free(jid);
 
     if (state) {
         stanza_attach_state(ctx, message, state);
     }
 
-    if (prefs_get_boolean(PREF_RECEIPTS_REQUEST)) {
+    if (oob_url) {
+        stanza_attach_x_oob_url(ctx, message, oob_url);
+    }
+
+    if (request_receipt) {
         stanza_attach_receipt_request(ctx, message);
     }
 
-    xmpp_send(conn, message);
+    _send_message_stanza(message);
     xmpp_stanza_release(message);
 
     return id;
 }
 
-char *
-message_send_chat_pgp(const char * const barejid, const char * const msg)
+char*
+message_send_chat_pgp(const char *const barejid, const char *const msg, gboolean request_receipt)
 {
-    xmpp_conn_t * const conn = connection_get_conn();
     xmpp_ctx_t * const ctx = connection_get_ctx();
 
-    char *state = _session_state(barejid);
-    char *jid = _session_jid(barejid);
+    char *state = chat_session_get_state(barejid);
+    char *jid = chat_session_get_jid(barejid);
     char *id = create_unique_id("msg");
 
     xmpp_stanza_t *message = NULL;
 #ifdef HAVE_LIBGPGME
-    char *account_name = jabber_get_account_name();
+    char *account_name = session_get_account_name();
     ProfAccount *account = accounts_get_account(account_name);
     if (account->pgp_keyid) {
         Jid *jidp = jid_create(jid);
-        char *encrypted = p_gpg_encrypt(jidp->barejid, msg);
+        char *encrypted = p_gpg_encrypt(jidp->barejid, msg, account->pgp_keyid);
         if (encrypted) {
-            message = stanza_create_message(ctx, id, jid, STANZA_TYPE_CHAT, "This message is encrypted.");
+            message = xmpp_message_new(ctx, STANZA_TYPE_CHAT, jid, id);
+            xmpp_message_set_body(message, "This message is encrypted.");
             xmpp_stanza_t *x = xmpp_stanza_new(ctx);
             xmpp_stanza_set_name(x, STANZA_NAME_X);
             xmpp_stanza_set_ns(x, STANZA_NS_ENCRYPTED);
@@ -168,15 +193,18 @@ message_send_chat_pgp(const char * const barejid, const char * const msg)
             xmpp_stanza_release(x);
             free(encrypted);
         } else {
-            message = stanza_create_message(ctx, id, jid, STANZA_TYPE_CHAT, msg);
+            message = xmpp_message_new(ctx, STANZA_TYPE_CHAT, jid, id);
+            xmpp_message_set_body(message, msg);
         }
         jid_destroy(jidp);
     } else {
-        message = stanza_create_message(ctx, id, jid, STANZA_TYPE_CHAT, msg);
+        message = xmpp_message_new(ctx, STANZA_TYPE_CHAT, jid, id);
+        xmpp_message_set_body(message, msg);
     }
     account_free(account);
 #else
-    message = stanza_create_message(ctx, id, jid, STANZA_TYPE_CHAT, msg);
+    message = xmpp_message_new(ctx, STANZA_TYPE_CHAT, jid, id);
+    xmpp_message_set_body(message, msg);
 #endif
     free(jid);
 
@@ -184,29 +212,28 @@ message_send_chat_pgp(const char * const barejid, const char * const msg)
         stanza_attach_state(ctx, message, state);
     }
 
-    stanza_attach_carbons_private(ctx, message);
-
-    if (prefs_get_boolean(PREF_RECEIPTS_REQUEST)) {
+    if (request_receipt) {
         stanza_attach_receipt_request(ctx, message);
     }
 
-    xmpp_send(conn, message);
+    _send_message_stanza(message);
     xmpp_stanza_release(message);
 
     return id;
 }
 
-char *
-message_send_chat_otr(const char * const barejid, const char * const msg)
+char*
+message_send_chat_otr(const char *const barejid, const char *const msg, gboolean request_receipt)
 {
-    xmpp_conn_t * const conn = connection_get_conn();
     xmpp_ctx_t * const ctx = connection_get_ctx();
 
-    char *state = _session_state(barejid);
-    char *jid = _session_jid(barejid);
+    char *state = chat_session_get_state(barejid);
+    char *jid = chat_session_get_jid(barejid);
     char *id = create_unique_id("msg");
 
-    xmpp_stanza_t *message = stanza_create_message(ctx, id, barejid, STANZA_TYPE_CHAT, msg);
+    xmpp_stanza_t *message = xmpp_message_new(ctx, STANZA_TYPE_CHAT, barejid, id);
+    xmpp_message_set_body(message, msg);
+
     free(jid);
 
     if (state) {
@@ -217,62 +244,71 @@ message_send_chat_otr(const char * const barejid, const char * const msg)
     stanza_attach_hints_no_copy(ctx, message);
     stanza_attach_hints_no_store(ctx, message);
 
-    if (prefs_get_boolean(PREF_RECEIPTS_REQUEST)) {
+    if (request_receipt) {
         stanza_attach_receipt_request(ctx, message);
     }
 
-    xmpp_send(conn, message);
+    _send_message_stanza(message);
     xmpp_stanza_release(message);
 
     return id;
 }
 
 void
-message_send_private(const char * const fulljid, const char * const msg)
+message_send_private(const char *const fulljid, const char *const msg, const char *const oob_url)
 {
-    xmpp_conn_t * const conn = connection_get_conn();
     xmpp_ctx_t * const ctx = connection_get_ctx();
     char *id = create_unique_id("prv");
-    xmpp_stanza_t *message = stanza_create_message(ctx, id, fulljid, STANZA_TYPE_CHAT, msg);
+
+    xmpp_stanza_t *message = xmpp_message_new(ctx, STANZA_TYPE_CHAT, fulljid, id);
+    xmpp_message_set_body(message, msg);
+
     free(id);
 
-    xmpp_send(conn, message);
+    if (oob_url) {
+        stanza_attach_x_oob_url(ctx, message, oob_url);
+    }
+
+    _send_message_stanza(message);
     xmpp_stanza_release(message);
 }
 
 void
-message_send_groupchat(const char * const roomjid, const char * const msg)
+message_send_groupchat(const char *const roomjid, const char *const msg, const char *const oob_url)
 {
-    xmpp_conn_t * const conn = connection_get_conn();
     xmpp_ctx_t * const ctx = connection_get_ctx();
     char *id = create_unique_id("muc");
-    xmpp_stanza_t *message = stanza_create_message(ctx, id, roomjid, STANZA_TYPE_GROUPCHAT, msg);
+
+    xmpp_stanza_t *message = xmpp_message_new(ctx, STANZA_TYPE_GROUPCHAT, roomjid, id);
+    xmpp_message_set_body(message, msg);
+
     free(id);
 
-    xmpp_send(conn, message);
+    if (oob_url) {
+        stanza_attach_x_oob_url(ctx, message, oob_url);
+    }
+
+    _send_message_stanza(message);
     xmpp_stanza_release(message);
 }
 
 void
-message_send_groupchat_subject(const char * const roomjid, const char * const subject)
+message_send_groupchat_subject(const char *const roomjid, const char *const subject)
 {
-    xmpp_conn_t * const conn = connection_get_conn();
     xmpp_ctx_t * const ctx = connection_get_ctx();
     xmpp_stanza_t *message = stanza_create_room_subject_message(ctx, roomjid, subject);
 
-    xmpp_send(conn, message);
+    _send_message_stanza(message);
     xmpp_stanza_release(message);
 }
 
 void
-message_send_invite(const char * const roomjid, const char * const contact,
-    const char * const reason)
+message_send_invite(const char *const roomjid, const char *const contact,
+    const char *const reason)
 {
-    xmpp_conn_t * const conn = connection_get_conn();
     xmpp_ctx_t * const ctx = connection_get_ctx();
     xmpp_stanza_t *stanza;
 
-
     muc_member_type_t member_type = muc_member_type(roomjid);
     if (member_type == MUC_MEMBER_TYPE_PUBLIC) {
         log_debug("Sending direct invite to %s, for %s", contact, roomjid);
@@ -283,62 +319,58 @@ message_send_invite(const char * const roomjid, const char * const contact,
         stanza = stanza_create_mediated_invite(ctx, roomjid, contact, reason);
     }
 
-    xmpp_send(conn, stanza);
+    _send_message_stanza(stanza);
     xmpp_stanza_release(stanza);
 }
 
 void
-message_send_composing(const char * const jid)
+message_send_composing(const char *const jid)
 {
-    xmpp_conn_t * const conn = connection_get_conn();
     xmpp_ctx_t * const ctx = connection_get_ctx();
 
     xmpp_stanza_t *stanza = stanza_create_chat_state(ctx, jid, STANZA_NAME_COMPOSING);
-    xmpp_send(conn, stanza);
+    _send_message_stanza(stanza);
     xmpp_stanza_release(stanza);
 
 }
 
 void
-message_send_paused(const char * const jid)
+message_send_paused(const char *const jid)
 {
-    xmpp_conn_t * const conn = connection_get_conn();
     xmpp_ctx_t * const ctx = connection_get_ctx();
     xmpp_stanza_t *stanza = stanza_create_chat_state(ctx, jid, STANZA_NAME_PAUSED);
-    xmpp_send(conn, stanza);
+    _send_message_stanza(stanza);
     xmpp_stanza_release(stanza);
 }
 
 void
-message_send_inactive(const char * const jid)
+message_send_inactive(const char *const jid)
 {
-    xmpp_conn_t * const conn = connection_get_conn();
     xmpp_ctx_t * const ctx = connection_get_ctx();
     xmpp_stanza_t *stanza = stanza_create_chat_state(ctx, jid, STANZA_NAME_INACTIVE);
 
-    xmpp_send(conn, stanza);
+    _send_message_stanza(stanza);
     xmpp_stanza_release(stanza);
 }
 
 void
-message_send_gone(const char * const jid)
+message_send_gone(const char *const jid)
 {
-    xmpp_conn_t * const conn = connection_get_conn();
     xmpp_ctx_t * const ctx = connection_get_ctx();
     xmpp_stanza_t *stanza = stanza_create_chat_state(ctx, jid, STANZA_NAME_GONE);
-    xmpp_send(conn, stanza);
+    _send_message_stanza(stanza);
     xmpp_stanza_release(stanza);
 }
 
-static int
-_message_error_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza, void * const userdata)
+static void
+_handle_error(xmpp_stanza_t *const stanza)
 {
-    char *id = xmpp_stanza_get_id(stanza);
-    char *jid = xmpp_stanza_get_attribute(stanza, STANZA_ATTR_FROM);
+    const char *id = xmpp_stanza_get_id(stanza);
+    const char *jid = xmpp_stanza_get_from(stanza);
     xmpp_stanza_t *error_stanza = xmpp_stanza_get_child_by_name(stanza, STANZA_NAME_ERROR);
-    char *type = NULL;
+    const char *type = NULL;
     if (error_stanza) {
-        type = xmpp_stanza_get_attribute(error_stanza, STANZA_ATTR_TYPE);
+        type = xmpp_stanza_get_type(error_stanza);
     }
 
     // stanza_get_error never returns NULL
@@ -376,37 +408,35 @@ _message_error_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza, v
     }
 
     free(err_msg);
-
-    return 1;
 }
 
-static int
-_muc_user_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza, void * const userdata)
+static void
+_handel_muc_user(xmpp_stanza_t *const stanza)
 {
     xmpp_ctx_t *ctx = connection_get_ctx();
     xmpp_stanza_t *xns_muc_user = xmpp_stanza_get_child_by_ns(stanza, STANZA_NS_MUC_USER);
-    char *room = xmpp_stanza_get_attribute(stanza, STANZA_ATTR_FROM);
+    const char *room = xmpp_stanza_get_from(stanza);
 
     if (!room) {
         log_warning("Message received with no from attribute, ignoring");
-        return 1;
+        return;
     }
 
     // XEP-0045
     xmpp_stanza_t *invite = xmpp_stanza_get_child_by_name(xns_muc_user, STANZA_NAME_INVITE);
     if (!invite) {
-        return 1;
+        return;
     }
 
-    char *invitor_jid = xmpp_stanza_get_attribute(invite, STANZA_ATTR_FROM);
+    const char *invitor_jid = xmpp_stanza_get_from(invite);
     if (!invitor_jid) {
         log_warning("Chat room invite received with no from attribute");
-        return 1;
+        return;
     }
 
     Jid *jidp = jid_create(invitor_jid);
     if (!jidp) {
-        return 1;
+        return;
     }
     char *invitor = jidp->barejid;
 
@@ -430,76 +460,65 @@ _muc_user_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza, void *
     if (password) {
         xmpp_free(ctx, password);
     }
-
-    return 1;
 }
 
-static int
-_conference_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza, void * const userdata)
+static void
+_handle_conference(xmpp_stanza_t *const stanza)
 {
     xmpp_stanza_t *xns_conference = xmpp_stanza_get_child_by_ns(stanza, STANZA_NS_CONFERENCE);
 
-    char *from = xmpp_stanza_get_attribute(stanza, STANZA_ATTR_FROM);
+    const char *from = xmpp_stanza_get_from(stanza);
     if (!from) {
         log_warning("Message received with no from attribute, ignoring");
-        return 1;
+        return;
     }
 
     Jid *jidp = jid_create(from);
     if (!jidp) {
-        return 1;
+        return;
     }
 
     // XEP-0249
-    char *room = xmpp_stanza_get_attribute(xns_conference, STANZA_ATTR_JID);
+    const char *room = xmpp_stanza_get_attribute(xns_conference, STANZA_ATTR_JID);
     if (!room) {
         jid_destroy(jidp);
-        return 1;
+        return;
     }
 
-    char *reason = xmpp_stanza_get_attribute(xns_conference, STANZA_ATTR_REASON);
-    char *password = xmpp_stanza_get_attribute(xns_conference, STANZA_ATTR_PASSWORD);
+    const char *reason = xmpp_stanza_get_attribute(xns_conference, STANZA_ATTR_REASON);
+    const char *password = xmpp_stanza_get_attribute(xns_conference, STANZA_ATTR_PASSWORD);
 
     sv_ev_room_invite(INVITE_DIRECT, jidp->barejid, room, reason, password);
     jid_destroy(jidp);
-
-    return 1;
 }
 
-static int
-_captcha_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza, void * const userdata)
+static void
+_handle_captcha(xmpp_stanza_t *const stanza)
 {
     xmpp_ctx_t *ctx = connection_get_ctx();
-    char *from = xmpp_stanza_get_attribute(stanza, STANZA_ATTR_FROM);
+    const char *from = xmpp_stanza_get_from(stanza);
 
     if (!from) {
         log_warning("Message received with no from attribute, ignoring");
-        return 1;
+        return;
     }
 
     // XEP-0158
-    xmpp_stanza_t *body = xmpp_stanza_get_child_by_name(stanza, STANZA_NAME_BODY);
-    if (!body) {
-        return 1;
-    }
-
-    char *message = xmpp_stanza_get_text(body);
+    char *message = xmpp_message_get_body(stanza);
     if (!message) {
-        return 1;
+        return;
     }
 
     sv_ev_room_broadcast(from, message);
     xmpp_free(ctx, message);
-
-    return 1;
 }
 
-static int
-_groupchat_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza, void * const userdata)
+static void
+_handle_groupchat(xmpp_stanza_t *const stanza)
 {
     xmpp_ctx_t *ctx = connection_get_ctx();
     char *message = NULL;
-    char *room_jid = xmpp_stanza_get_attribute(stanza, STANZA_ATTR_FROM);
+    const char *room_jid = xmpp_stanza_get_from(stanza);
     Jid *jid = jid_create(room_jid);
 
     // handle room subject
@@ -510,55 +529,41 @@ _groupchat_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza, void
         xmpp_free(ctx, message);
 
         jid_destroy(jid);
-        return 1;
+        return;
     }
 
     // handle room broadcasts
     if (!jid->resourcepart) {
-        xmpp_stanza_t *body = xmpp_stanza_get_child_by_name(stanza, STANZA_NAME_BODY);
-        if (!body) {
-            jid_destroy(jid);
-            return 1;
-        }
-
-        message = xmpp_stanza_get_text(body);
+        message = xmpp_message_get_body(stanza);
         if (!message) {
             jid_destroy(jid);
-            return 1;
+            return;
         }
 
         sv_ev_room_broadcast(room_jid, message);
         xmpp_free(ctx, message);
 
         jid_destroy(jid);
-        return 1;
+        return;
     }
 
     if (!jid_is_valid_room_form(jid)) {
         log_error("Invalid room JID: %s", jid->str);
         jid_destroy(jid);
-        return 1;
+        return;
     }
 
     // room not active in profanity
     if (!muc_active(jid->barejid)) {
         log_error("Message received for inactive chat room: %s", jid->str);
         jid_destroy(jid);
-        return 1;
+        return;
     }
 
-    xmpp_stanza_t *body = xmpp_stanza_get_child_by_name(stanza, STANZA_NAME_BODY);
-
-    // check for and deal with message
-    if (!body) {
-        jid_destroy(jid);
-        return 1;
-    }
-
-    message = xmpp_stanza_get_text(body);
+    message = xmpp_message_get_body(stanza);
     if (!message) {
         jid_destroy(jid);
-        return 1;
+        return;
     }
 
     // determine if the notifications happened whilst offline
@@ -572,68 +577,61 @@ _groupchat_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza, void
 
     xmpp_free(ctx, message);
     jid_destroy(jid);
-
-    return 1;
 }
 
 void
-_message_send_receipt(const char * const fulljid, const char * const message_id)
+_message_send_receipt(const char *const fulljid, const char *const message_id)
 {
-    xmpp_conn_t * const conn = connection_get_conn();
     xmpp_ctx_t * const ctx = connection_get_ctx();
-    xmpp_stanza_t *message = xmpp_stanza_new(ctx);
-    xmpp_stanza_set_name(message, STANZA_NAME_MESSAGE);
+
     char *id = create_unique_id("receipt");
-    xmpp_stanza_set_id(message, id);
+    xmpp_stanza_t *message = xmpp_message_new(ctx, NULL, fulljid, id);
     free(id);
-    xmpp_stanza_set_attribute(message, STANZA_ATTR_TO, fulljid);
 
     xmpp_stanza_t *receipt = xmpp_stanza_new(ctx);
     xmpp_stanza_set_name(receipt, "received");
     xmpp_stanza_set_ns(receipt, STANZA_NS_RECEIPTS);
-    xmpp_stanza_set_attribute(receipt, STANZA_ATTR_ID, message_id);
+    xmpp_stanza_set_id(receipt, message_id);
 
     xmpp_stanza_add_child(message, receipt);
     xmpp_stanza_release(receipt);
 
-    xmpp_send(conn, message);
+    _send_message_stanza(message);
     xmpp_stanza_release(message);
 }
 
-static int
-_receipt_received_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza, void * const userdata)
+static void
+_handle_receipt_received(xmpp_stanza_t *const stanza)
 {
     xmpp_stanza_t *receipt = xmpp_stanza_get_child_by_ns(stanza, STANZA_NS_RECEIPTS);
-    char *name = xmpp_stanza_get_name(receipt);
+    const char *name = xmpp_stanza_get_name(receipt);
     if (g_strcmp0(name, "received") != 0) {
-        return 1;
+        return;
     }
 
-    char *id = xmpp_stanza_get_attribute(receipt, STANZA_ATTR_ID);
+    const char *id = xmpp_stanza_get_id(receipt);
     if (!id) {
-        return 1;
+        return;
     }
 
-    char *fulljid = xmpp_stanza_get_attribute(stanza, STANZA_ATTR_FROM);
+    const char *fulljid = xmpp_stanza_get_from(stanza);
     if (!fulljid) {
-        return 1;
+        return;
     }
 
     Jid *jidp = jid_create(fulljid);
     sv_ev_message_receipt(jidp->barejid, id);
     jid_destroy(jidp);
-
-    return 1;
 }
 
 void
-_receipt_request_handler(xmpp_stanza_t * const stanza)
+_receipt_request_handler(xmpp_stanza_t *const stanza)
 {
     if (!prefs_get_boolean(PREF_RECEIPTS_SEND)) {
         return;
     }
 
-    char *id = xmpp_stanza_get_id(stanza);
+    const char *id = xmpp_stanza_get_id(stanza);
     if (!id) {
         return;
     }
@@ -643,26 +641,21 @@ _receipt_request_handler(xmpp_stanza_t * const stanza)
         return;
     }
 
-    char *receipts_name = xmpp_stanza_get_name(receipts);
+    const char *receipts_name = xmpp_stanza_get_name(receipts);
     if (g_strcmp0(receipts_name, "request") != 0) {
         return;
     }
 
-    gchar *from = xmpp_stanza_get_attribute(stanza, STANZA_ATTR_FROM);
+    const gchar *from = xmpp_stanza_get_from(stanza);
     Jid *jid = jid_create(from);
     _message_send_receipt(jid->fulljid, id);
     jid_destroy(jid);
 }
 
 void
-_private_chat_handler(xmpp_stanza_t * const stanza, const char * const fulljid)
+_private_chat_handler(xmpp_stanza_t *const stanza, const char *const fulljid)
 {
-    xmpp_stanza_t *body = xmpp_stanza_get_child_by_name(stanza, STANZA_NAME_BODY);
-    if (!body) {
-        return;
-    }
-
-    char *message = xmpp_stanza_get_text(body);
+    char *message = xmpp_message_get_body(stanza);
     if (!message) {
         return;
     }
@@ -680,77 +673,104 @@ _private_chat_handler(xmpp_stanza_t * const stanza, const char * const fulljid)
 }
 
 static gboolean
-_handle_carbons(xmpp_stanza_t * const stanza)
+_handle_carbons(xmpp_stanza_t *const stanza)
 {
     xmpp_stanza_t *carbons = xmpp_stanza_get_child_by_ns(stanza, STANZA_NS_CARBONS);
     if (!carbons) {
         return FALSE;
     }
 
-    char *name = xmpp_stanza_get_name(carbons);
-    if ((g_strcmp0(name, "received") == 0) || (g_strcmp0(name, "sent")) == 0) {
-        xmpp_stanza_t *forwarded = xmpp_stanza_get_child_by_ns(carbons, STANZA_NS_FORWARD);
-        xmpp_stanza_t *message = xmpp_stanza_get_child_by_name(forwarded, STANZA_NAME_MESSAGE);
-
-        xmpp_ctx_t *ctx = connection_get_ctx();
-
-        gchar *to = xmpp_stanza_get_attribute(message, STANZA_ATTR_TO);
-        gchar *from = xmpp_stanza_get_attribute(message, STANZA_ATTR_FROM);
-
-        // happens when receive a carbon of a self sent message
-        if (!to) to = from;
-
-        Jid *jid_from = jid_create(from);
-        Jid *jid_to = jid_create(to);
-        Jid *my_jid = jid_create(jabber_get_fulljid());
-
-        // check for and deal with message
-        xmpp_stanza_t *body = xmpp_stanza_get_child_by_name(message, STANZA_NAME_BODY);
-        if (body) {
-            char *message = xmpp_stanza_get_text(body);
-            if (message) {
-                // if we are the recipient, treat as standard incoming message
-                if(g_strcmp0(my_jid->barejid, jid_to->barejid) == 0){
-                    sv_ev_incoming_carbon(jid_from->barejid, jid_from->resourcepart, message);
-                }
-                // else treat as a sent message
-                else{
-                    sv_ev_outgoing_carbon(jid_to->barejid, message);
-                }
-                xmpp_free(ctx, message);
-            }
-        }
-
-        jid_destroy(jid_from);
-        jid_destroy(jid_to);
-        jid_destroy(my_jid);
-
+    const char *name = xmpp_stanza_get_name(carbons);
+    if (!name) {
+        log_error("Unable to retrieve stanza name for Carbon");
         return TRUE;
     }
 
-    return FALSE;
+    if (g_strcmp0(name, "private") == 0) {
+        log_info("Carbon received with private element.");
+        return FALSE;
+    }
+
+    if ((g_strcmp0(name, "received") != 0) && (g_strcmp0(name, "sent") != 0)) {
+        log_warning("Carbon received with unrecognised stanza name: %s", name);
+        return TRUE;
+    }
+
+    xmpp_stanza_t *forwarded = xmpp_stanza_get_child_by_ns(carbons, STANZA_NS_FORWARD);
+    if (!forwarded) {
+        log_warning("Carbon received with no forwarded element");
+        return TRUE;
+    }
+
+    xmpp_stanza_t *message = xmpp_stanza_get_child_by_name(forwarded, STANZA_NAME_MESSAGE);
+    if (!message) {
+        log_warning("Carbon received with no message element");
+        return TRUE;
+    }
+
+    char *message_txt = xmpp_message_get_body(message);
+    if (!message_txt) {
+        log_warning("Carbon received with no message.");
+        return TRUE;
+    }
+
+    const gchar *to = xmpp_stanza_get_to(message);
+    const gchar *from = xmpp_stanza_get_from(message);
+
+    // happens when receive a carbon of a self sent message
+    if (!to) to = from;
+
+    Jid *jid_from = jid_create(from);
+    Jid *jid_to = jid_create(to);
+    Jid *my_jid = jid_create(connection_get_fulljid());
+
+    // check for pgp encrypted message
+    char *enc_message = NULL;
+    xmpp_stanza_t *x = xmpp_stanza_get_child_by_ns(message, STANZA_NS_ENCRYPTED);
+    if (x) {
+        enc_message = xmpp_stanza_get_text(x);
+    }
+
+    // if we are the recipient, treat as standard incoming message
+    if (g_strcmp0(my_jid->barejid, jid_to->barejid) == 0) {
+        sv_ev_incoming_carbon(jid_from->barejid, jid_from->resourcepart, message_txt, enc_message);
+
+    // else treat as a sent message
+    } else {
+        sv_ev_outgoing_carbon(jid_to->barejid, message_txt, enc_message);
+    }
+
+    xmpp_ctx_t *ctx = connection_get_ctx();
+    xmpp_free(ctx, message_txt);
+    xmpp_free(ctx, enc_message);
+
+    jid_destroy(jid_from);
+    jid_destroy(jid_to);
+    jid_destroy(my_jid);
+
+    return TRUE;
 }
 
-static int
-_chat_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza, void * const userdata)
+static void
+_handle_chat(xmpp_stanza_t *const stanza)
 {
     // ignore if type not chat or absent
-    char *type = xmpp_stanza_get_type(stanza);
+    const char *type = xmpp_stanza_get_type(stanza);
     if (!(g_strcmp0(type, "chat") == 0 || type == NULL)) {
-        return 1;
+        return;
     }
 
     // check if carbon message
     gboolean res = _handle_carbons(stanza);
     if (res) {
-        return 1;
+        return;
     }
 
     // ignore handled namespaces
     xmpp_stanza_t *conf = xmpp_stanza_get_child_by_ns(stanza, STANZA_NS_CONFERENCE);
     xmpp_stanza_t *captcha = xmpp_stanza_get_child_by_ns(stanza, STANZA_NS_CAPTCHA);
     if (conf || captcha) {
-        return 1;
+        return;
     }
 
     // some clients send the mucuser namespace with private messages
@@ -759,17 +779,17 @@ _chat_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza, void * con
     xmpp_stanza_t *mucuser = xmpp_stanza_get_child_by_ns(stanza, STANZA_NS_MUC_USER);
     xmpp_stanza_t *body = xmpp_stanza_get_child_by_name(stanza, STANZA_NAME_BODY);
     if (mucuser && body == NULL) {
-        return 1;
+        return;
     }
 
-    gchar *from = xmpp_stanza_get_attribute(stanza, STANZA_ATTR_FROM);
+    const gchar *from = xmpp_stanza_get_from(stanza);
     Jid *jid = jid_create(from);
 
     // private message from chat room use full jid (room/nick)
     if (muc_active(jid->barejid)) {
         _private_chat_handler(stanza, jid->fulljid);
         jid_destroy(jid);
-        return 1;
+        return;
     }
 
     // standard chat message, use jid without resource
@@ -815,5 +835,22 @@ _chat_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza, void * con
 
     if (timestamp) g_date_time_unref(timestamp);
     jid_destroy(jid);
-    return 1;
+}
+
+static void
+_send_message_stanza(xmpp_stanza_t *const stanza)
+{
+    char *text;
+    size_t text_size;
+    xmpp_stanza_to_text(stanza, &text, &text_size);
+
+    xmpp_conn_t *conn = connection_get_conn();
+    char *plugin_text = plugins_on_message_stanza_send(text);
+    if (plugin_text) {
+        xmpp_send_raw_string(conn, "%s", plugin_text);
+        free(plugin_text);
+    } else {
+        xmpp_send_raw_string(conn, "%s", text);
+    }
+    xmpp_free(connection_get_ctx(), text);
 }
