@@ -1,7 +1,7 @@
 /*
  * roster.c
  *
- * Copyright (C) 2012 - 2015 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2016 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with Profanity.  If not, see <http://www.gnu.org/licenses/>.
+ * along with Profanity.  If not, see <https://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give permission to
  * link the code of portions of this program with the OpenSSL library under
@@ -32,107 +32,94 @@
  *
  */
 
+#include "config.h"
+
 #include <assert.h>
 #include <stdlib.h>
 #include <string.h>
 
 #include <glib.h>
-#include <strophe.h>
 
-#include "log.h"
+#ifdef HAVE_LIBMESODE
+#include <mesode.h>
+#endif
+
+#ifdef HAVE_LIBSTROPHE
+#include <strophe.h>
+#endif
+
 #include "profanity.h"
-#include "ui/ui.h"
+#include "log.h"
+#include "config/preferences.h"
+#include "plugins/plugins.h"
 #include "event/server_events.h"
 #include "event/client_events.h"
 #include "tools/autocomplete.h"
+#include "ui/ui.h"
+#include "xmpp/session.h"
+#include "xmpp/iq.h"
 #include "xmpp/connection.h"
 #include "xmpp/roster.h"
-#include "roster_list.h"
+#include "xmpp/roster_list.h"
 #include "xmpp/stanza.h"
 #include "xmpp/xmpp.h"
 
-#define HANDLE(type, func) xmpp_handler_add(conn, func, XMPP_NS_ROSTER, \
-STANZA_NAME_IQ, type, ctx)
-
 // callback data for group commands
 typedef struct _group_data {
     char *name;
     char *group;
 } GroupData;
 
-// event handlers
-static int _roster_set_handler(xmpp_conn_t * const conn,
-    xmpp_stanza_t * const stanza, void * const userdata);
-static int _roster_result_handler(xmpp_conn_t * const conn,
-    xmpp_stanza_t * const stanza, void * const userdata);
-
 // id handlers
-static int
-_group_add_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza,
-    void * const userdata);
-static int
-_group_remove_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza,
-    void * const userdata);
+static int _group_add_id_handler(xmpp_stanza_t *const stanza, void *const userdata);
+static int _group_remove_id_handler(xmpp_stanza_t *const stanza, void *const userdata);
+static void _free_group_data(GroupData *data);
 
 // helper functions
-GSList * _get_groups_from_item(xmpp_stanza_t *item);
-
-void
-roster_add_handlers(void)
-{
-    xmpp_conn_t * const conn = connection_get_conn();
-    xmpp_ctx_t * const ctx = connection_get_ctx();
-
-    HANDLE(STANZA_TYPE_SET,    _roster_set_handler);
-    HANDLE(STANZA_TYPE_RESULT, _roster_result_handler);
-}
+GSList* _get_groups_from_item(xmpp_stanza_t *item);
 
 void
 roster_request(void)
 {
-    xmpp_conn_t * const conn = connection_get_conn();
     xmpp_ctx_t * const ctx = connection_get_ctx();
     xmpp_stanza_t *iq = stanza_create_roster_iq(ctx);
-    xmpp_send(conn, iq);
+    iq_send_stanza(iq);
     xmpp_stanza_release(iq);
 }
 
 void
-roster_send_add_new(const char * const barejid, const char * const name)
+roster_send_add_new(const char *const barejid, const char *const name)
 {
-    xmpp_conn_t * const conn = connection_get_conn();
     xmpp_ctx_t * const ctx = connection_get_ctx();
     char *id = create_unique_id("roster");
     xmpp_stanza_t *iq = stanza_create_roster_set(ctx, id, barejid, name, NULL);
     free(id);
-    xmpp_send(conn, iq);
+    iq_send_stanza(iq);
     xmpp_stanza_release(iq);
 }
 
 void
-roster_send_remove(const char * const barejid)
+roster_send_remove(const char *const barejid)
 {
-    xmpp_conn_t * const conn = connection_get_conn();
     xmpp_ctx_t * const ctx = connection_get_ctx();
     xmpp_stanza_t *iq = stanza_create_roster_remove_set(ctx, barejid);
-    xmpp_send(conn, iq);
+    iq_send_stanza(iq);
     xmpp_stanza_release(iq);
 }
 
 void
-roster_send_name_change(const char * const barejid, const char * const new_name, GSList *groups)
+roster_send_name_change(const char *const barejid, const char *const new_name, GSList *groups)
 {
-    xmpp_conn_t * const conn = connection_get_conn();
     xmpp_ctx_t * const ctx = connection_get_ctx();
     char *id = create_unique_id("roster");
     xmpp_stanza_t *iq = stanza_create_roster_set(ctx, id, barejid, new_name, groups);
     free(id);
-    xmpp_send(conn, iq);
+    iq_send_stanza(iq);
     xmpp_stanza_release(iq);
 }
 
 void
-roster_send_add_to_group(const char * const group, PContact contact)
+roster_send_add_to_group(const char *const group, PContact contact)
 {
     GSList *groups = p_contact_groups(contact);
     GSList *new_groups = NULL;
@@ -152,32 +139,28 @@ roster_send_add_to_group(const char * const group, PContact contact)
         data->name = strdup(p_contact_barejid(contact));
     }
 
-    xmpp_conn_t * const conn = connection_get_conn();
     xmpp_ctx_t * const ctx = connection_get_ctx();
-    xmpp_id_handler_add(conn, _group_add_handler, unique_id, data);
+    iq_id_handler_add(unique_id, _group_add_id_handler, (ProfIdFreeCallback)_free_group_data, data);
     xmpp_stanza_t *iq = stanza_create_roster_set(ctx, unique_id, p_contact_barejid(contact),
         p_contact_name(contact), new_groups);
-    xmpp_send(conn, iq);
+    iq_send_stanza(iq);
     xmpp_stanza_release(iq);
     free(unique_id);
 }
 
 static int
-_group_add_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza,
-    void * const userdata)
+_group_add_id_handler(xmpp_stanza_t *const stanza, void *const userdata)
 {
     if (userdata) {
         GroupData *data = userdata;
         ui_group_added(data->name, data->group);
-        free(data->name);
-        free(data->group);
-        free(userdata);
+        _free_group_data(data);
     }
     return 0;
 }
 
 void
-roster_send_remove_from_group(const char * const group, PContact contact)
+roster_send_remove_from_group(const char *const group, PContact contact)
 {
     GSList *groups = p_contact_groups(contact);
     GSList *new_groups = NULL;
@@ -188,7 +171,6 @@ roster_send_remove_from_group(const char * const group, PContact contact)
         groups = g_slist_next(groups);
     }
 
-    xmpp_conn_t * const conn = connection_get_conn();
     xmpp_ctx_t * const ctx = connection_get_ctx();
 
     // add an id handler to handle the response
@@ -201,17 +183,16 @@ roster_send_remove_from_group(const char * const group, PContact contact)
         data->name = strdup(p_contact_barejid(contact));
     }
 
-    xmpp_id_handler_add(conn, _group_remove_handler, unique_id, data);
+    iq_id_handler_add(unique_id, _group_remove_id_handler, (ProfIdFreeCallback)_free_group_data, data);
     xmpp_stanza_t *iq = stanza_create_roster_set(ctx, unique_id, p_contact_barejid(contact),
         p_contact_name(contact), new_groups);
-    xmpp_send(conn, iq);
+    iq_send_stanza(iq);
     xmpp_stanza_release(iq);
     free(unique_id);
 }
 
 static int
-_group_remove_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza,
-    void * const userdata)
+_group_remove_id_handler(xmpp_stanza_t *const stanza, void *const userdata)
 {
     if (userdata) {
         GroupData *data = userdata;
@@ -223,9 +204,8 @@ _group_remove_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza,
     return 0;
 }
 
-static int
-_roster_set_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza,
-    void * const userdata)
+void
+roster_set_handler(xmpp_stanza_t *const stanza)
 {
     xmpp_stanza_t *query =
         xmpp_stanza_get_child_by_name(stanza, STANZA_NAME_QUERY);
@@ -233,15 +213,15 @@ _roster_set_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza,
         xmpp_stanza_get_child_by_name(query, STANZA_NAME_ITEM);
 
     if (item == NULL) {
-        return 1;
+        return;
     }
 
     // if from attribute exists and it is not current users barejid, ignore push
-    Jid *my_jid = jid_create(jabber_get_fulljid());
-    const char *from = xmpp_stanza_get_attribute(stanza, STANZA_ATTR_FROM);
+    Jid *my_jid = jid_create(connection_get_fulljid());
+    const char *from = xmpp_stanza_get_from(stanza);
     if (from && (strcmp(from, my_jid->barejid) != 0)) {
         jid_destroy(my_jid);
-        return 1;
+        return;
     }
     jid_destroy(my_jid);
 
@@ -290,16 +270,16 @@ _roster_set_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza,
 
     g_free(barejid_lower);
 
-    return 1;
+    return;
 }
 
-static int
-_roster_result_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza, void * const userdata)
+void
+roster_result_handler(xmpp_stanza_t *const stanza)
 {
-    const char *id = xmpp_stanza_get_attribute(stanza, STANZA_ATTR_ID);
+    const char *id = xmpp_stanza_get_id(stanza);
 
     if (g_strcmp0(id, "roster") != 0) {
-        return 1;
+        return;
     }
 
     // handle initial roster response
@@ -334,13 +314,10 @@ _roster_result_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza, v
 
     sv_ev_roster_received();
 
-    resource_presence_t conn_presence = accounts_get_login_presence(jabber_get_account_name());
-    cl_ev_presence_send(conn_presence, NULL, 0);
-
-    return 1;
+    return;
 }
 
-GSList *
+GSList*
 _get_groups_from_item(xmpp_stanza_t *item)
 {
     GSList *groups = NULL;
@@ -358,3 +335,13 @@ _get_groups_from_item(xmpp_stanza_t *item)
 
     return groups;
 }
+
+static void
+_free_group_data(GroupData *data)
+{
+    if (data) {
+        free(data->group);
+        free(data->name);
+        free(data);
+    }
+}
