@@ -1,7 +1,7 @@
 /*
  * roster.h
  *
- * Copyright (C) 2012 - 2015 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2016 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with Profanity.  If not, see <http://www.gnu.org/licenses/>.
+ * along with Profanity.  If not, see <https://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give permission to
  * link the code of portions of this program with the OpenSSL library under
@@ -35,7 +35,8 @@
 #ifndef XMPP_ROSTER_H
 #define XMPP_ROSTER_H
 
-void roster_add_handlers(void);
 void roster_request(void);
+void roster_set_handler(xmpp_stanza_t *const stanza);
+void roster_result_handler(xmpp_stanza_t *const stanza);
 
 #endif
