@@ -15,10 +15,10 @@ void
 rooms_query(void **state)
 {
     stbbr_for_id("confreq",
-        "<iq id=\"confreq\" type=\"result\" to=\"stabber@localhost/profanity\" from=\"conference.localhost\">"
-            "<query xmlns=\"http://jabber.org/protocol/disco#items\">"
-                "<item jid=\"chatroom@conference.localhost\" name=\"A chat room\"/>"
-                "<item jid=\"hangout@conference.localhost\" name=\"Another chat room\"/>"
+        "<iq id='confreq' type='result' to='stabber@localhost/profanity' from='conference.localhost'>"
+            "<query xmlns='http://jabber.org/protocol/disco#items'>"
+                "<item jid='chatroom@conference.localhost' name='A chat room'/>"
+                "<item jid='hangout@conference.localhost' name='Another chat room'/>"
             "</query>"
         "</iq>"
     );
@@ -31,8 +31,8 @@ rooms_query(void **state)
     assert_true(prof_output_exact("hangout@conference.localhost, (Another chat room)"));
 
     assert_true(stbbr_last_received(
-        "<iq id=\"confreq\" to=\"conference.localhost\" type=\"get\">"
-            "<query xmlns=\"http://jabber.org/protocol/disco#items\"/>"
+        "<iq id='confreq' to='conference.localhost' type='get'>"
+            "<query xmlns='http://jabber.org/protocol/disco#items'/>"
         "</iq>"
     ));
 }
