@@ -1,3 +1,5 @@
+#include "config.h"
+
 #include <glib.h>
 #include <wchar.h>
 
@@ -6,6 +8,9 @@
 
 #include "ui/window.h"
 #include "ui/ui.h"
+#ifdef HAVE_LIBOTR
+#include "otr/otr.h"
+#endif
 
 #include "tests/unittests/ui/stub_ui.h"
 
@@ -59,30 +64,17 @@ void ui_update(void) {}
 void ui_close(void) {}
 void ui_redraw(void) {}
 void ui_resize(void) {}
-GSList* ui_get_chat_recipients(void)
-{
-    return NULL;
-}
 
-void ui_switch_win(ProfWin *win) {}
+void ui_focus_win(ProfWin *win) {}
 
-void ui_gone_secure(const char * const barejid, gboolean trusted) {}
-void ui_gone_insecure(const char * const barejid) {}
-void ui_trust(const char * const barejid) {}
-void ui_untrust(const char * const barejid) {}
-void ui_smp_recipient_initiated(const char * const barejid) {}
-void ui_smp_recipient_initiated_q(const char * const barejid, const char *question) {}
+#ifdef HAVE_LIBOTR
+void chatwin_otr_secured(ProfChatWin *chatwin, gboolean trusted) {}
+void chatwin_otr_unsecured(ProfChatWin *chatwin) {}
+void chatwin_otr_trust(ProfChatWin *chatwin) {}
+void chatwin_otr_untrust(ProfChatWin *chatwin) {}
+void chatwin_otr_smp_event(ProfChatWin *chatwin, prof_otr_smp_event_t event, void *data) {}
+#endif
 
-void ui_smp_successful(const char * const barejid) {}
-void ui_smp_unsuccessful_sender(const char * const barejid) {}
-void ui_smp_unsuccessful_receiver(const char * const barejid) {}
-void ui_smp_aborted(const char * const barejid) {}
-
-void ui_smp_answer_success(const char * const barejid) {}
-void ui_smp_answer_failure(const char * const barejid) {}
-
-void ui_otr_authenticating(const char * const barejid) {}
-void ui_otr_authetication_waiting(const char * const recipient) {}
 void ui_sigwinch_handler(int sig) {}
 
 unsigned long ui_get_idle_time(void)
@@ -91,21 +83,13 @@ unsigned long ui_get_idle_time(void)
 }
 
 void ui_reset_idle_time(void) {}
-ProfPrivateWin* ui_new_private_win(const char * const fulljid)
-{
-    return NULL;
-}
 
-ProfChatWin* ui_new_chat_win(const char * const barejid)
+ProfChatWin* chatwin_new(const char * const barejid)
 {
     return NULL;
 }
 
 void ui_print_system_msg_from_recipient(const char * const barejid, const char *message) {}
-gint ui_unread(void)
-{
-    return 0;
-}
 
 void ui_close_connected_win(int index) {}
 int ui_close_all_wins(void)
@@ -144,12 +128,6 @@ void ui_current_print_formatted_line(const char show_char, int attrs, const char
 void ui_current_error_line(const char * const msg) {}
 void ui_win_error_line(ProfWin *window, const char * const msg) {}
 
-
-win_type_t ui_win_type(int index)
-{
-    return WIN_CONSOLE;
-}
-
 void ui_close_win(int index) {}
 
 int ui_win_unread(int index)
@@ -157,18 +135,17 @@ int ui_win_unread(int index)
     return 0;
 }
 
-void ui_page_up(void) {}
-void ui_page_down(void) {}
-void ui_subwin_page_up(void) {}
-void ui_subwin_page_down(void) {}
-void ui_clear_win(ProfWin *window) {}
-
 char * ui_ask_password(void)
 {
     return mock_ptr_type(char *);
 }
 
-void ui_handle_stanza(const char * const msg) {}
+char *ui_get_line(void)
+{
+    return NULL;
+}
+
+void xmlwin_show(ProfXMLWin *xmlwin, const char * const msg) {}
 
 // ui events
 void ui_contact_online(char *barejid, Resource *resource, GDateTime *last_activity)
@@ -179,68 +156,73 @@ void ui_contact_online(char *barejid, Resource *resource, GDateTime *last_activi
 }
 
 void ui_contact_typing(const char * const barejid, const char * const resource) {}
-void ui_incoming_msg(ProfChatWin *chatwin, const char * const resource, const char * const message, GDateTime *timestamp, gboolean win_created, prof_enc_t enc_mode) {}
-void ui_message_receipt(const char * const barejid, const char * const id) {}
+void chatwin_incoming_msg(ProfChatWin *chatwin, const char * const resource, const char * const message, GDateTime *timestamp, gboolean win_created, prof_enc_t enc_mode) {}
+void chatwin_receipt_received(ProfChatWin *chatwin, const char * const id) {}
 
-void ui_incoming_private_msg(const char * const fulljid, const char * const message, GDateTime *timestamp) {}
+void privwin_incoming_msg(ProfPrivateWin *privatewin, const char * const message, GDateTime *timestamp) {}
 
 void ui_disconnected(void) {}
-void ui_recipient_gone(const char * const barejid, const char * const resource) {}
+void chatwin_recipient_gone(ProfChatWin *chatwin) {}
 
-void ui_outgoing_chat_msg(ProfChatWin *chatwin, const char * const message, char *id, prof_enc_t enc_mode) {}
-void ui_outgoing_chat_msg_carbon(const char * const barejid, const char * const message) {}
-void ui_outgoing_private_msg(ProfPrivateWin *privwin, const char * const message) {}
+void chatwin_outgoing_msg(ProfChatWin *chatwin, const char * const message, char *id, prof_enc_t enc_mode,
+    gboolean request_receipt) {}
+void chatwin_outgoing_carbon(ProfChatWin *chatwin, const char * const message, prof_enc_t enc_mode) {}
+void privwin_outgoing_msg(ProfPrivateWin *privwin, const char * const message) {}
+
+void privwin_occupant_offline(ProfPrivateWin *privwin) {}
+void privwin_occupant_kicked(ProfPrivateWin *privwin, const char *const actor, const char *const reason) {}
+void privwin_occupant_banned(ProfPrivateWin *privwin, const char *const actor, const char *const reason) {}
+void privwin_occupant_online(ProfPrivateWin *privwin) {}
+void privwin_message_occupant_offline(ProfPrivateWin *privwin) {}
+
+void privwin_message_left_room(ProfPrivateWin *privwin) {}
 
 void ui_room_join(const char * const roomjid, gboolean focus) {}
 void ui_switch_to_room(const char * const roomjid) {}
 
-void ui_room_role_change(const char * const roomjid, const char * const role, const char * const actor,
+void mucwin_role_change(ProfMucWin *mucwin, const char * const role, const char * const actor,
     const char * const reason) {}
-void ui_room_affiliation_change(const char * const roomjid, const char * const affiliation, const char * const actor,
+void mucwin_affiliation_change(ProfMucWin *mucwin, const char * const affiliation, const char * const actor,
     const char * const reason) {}
-void ui_room_role_and_affiliation_change(const char * const roomjid, const char * const role,
+void mucwin_role_and_affiliation_change(ProfMucWin *mucwin, const char * const role,
     const char * const affiliation, const char * const actor, const char * const reason) {}
-void ui_room_occupant_role_change(const char * const roomjid, const char * const nick, const char * const role,
+void mucwin_occupant_role_change(ProfMucWin *mucwin, const char * const nick, const char * const role,
     const char * const actor, const char * const reason) {}
-void ui_room_occupant_affiliation_change(const char * const roomjid, const char * const nick, const char * const affiliation,
+void mucwin_occupant_affiliation_change(ProfMucWin *mucwin, const char * const nick, const char * const affiliation,
     const char * const actor, const char * const reason) {}
-void ui_room_occupant_role_and_affiliation_change(const char * const roomjid, const char * const nick, const char * const role,
+void mucwin_occupant_role_and_affiliation_change(ProfMucWin *mucwin, const char * const nick, const char * const role,
     const char * const affiliation, const char * const actor, const char * const reason) {}
-void ui_room_roster(const char * const roomjid, GList *occupants, const char * const presence) {}
-void ui_room_history(const char * const roomjid, const char * const nick,
-    GDateTime *timestamp, const char * const message) {}
-void ui_room_message(const char * const roomjid, const char * const nick,
-    const char * const message) {}
-void ui_room_subject(const char * const roomjid, const char * const nick, const char * const subject) {}
-void ui_room_requires_config(const char * const roomjid) {}
+void mucwin_roster(ProfMucWin *mucwin, GList *occupants, const char * const presence) {}
+void mucwin_history(ProfMucWin *mucwin, const char * const nick, GDateTime *timestamp, const char * const message) {}
+void mucwin_message(ProfMucWin *mucwin, const char *const nick, const char *const message, GSList *mentions, GList *triggers) {}
+void mucwin_subject(ProfMucWin *mucwin, const char * const nick, const char * const subject) {}
+void mucwin_requires_config(ProfMucWin *mucwin) {}
 void ui_room_destroy(const char * const roomjid) {}
-void ui_show_room_info(ProfMucWin *mucwin) {}
-void ui_show_room_role_list(ProfMucWin *mucwin, muc_role_t role) {}
-void ui_show_room_affiliation_list(ProfMucWin *mucwin, muc_affiliation_t affiliation) {}
-void ui_handle_room_info_error(const char * const roomjid, const char * const error) {}
-void ui_show_room_disco_info(const char * const roomjid, GSList *identities, GSList *features) {}
+void mucwin_info(ProfMucWin *mucwin) {}
+void mucwin_show_role_list(ProfMucWin *mucwin, muc_role_t role) {}
+void mucwin_show_affiliation_list(ProfMucWin *mucwin, muc_affiliation_t affiliation) {}
+void mucwin_room_info_error(ProfMucWin *mucwin, const char * const error) {}
+void mucwin_room_disco_info(ProfMucWin *mucwin, GSList *identities, GSList *features) {}
 void ui_room_destroyed(const char * const roomjid, const char * const reason, const char * const new_jid,
     const char * const password) {}
 void ui_room_kicked(const char * const roomjid, const char * const actor, const char * const reason) {}
-void ui_room_member_kicked(const char * const roomjid, const char * const nick, const char * const actor,
+void mucwin_occupant_kicked(ProfMucWin *mucwin, const char * const nick, const char * const actor,
     const char * const reason) {}
 void ui_room_banned(const char * const roomjid, const char * const actor, const char * const reason) {}
-void ui_room_member_banned(const char * const roomjid, const char * const nick, const char * const actor,
+void mucwin_occupant_banned(ProfMucWin *mucwin, const char * const nick, const char * const actor,
     const char * const reason) {}
 void ui_leave_room(const char * const roomjid) {}
-void ui_room_broadcast(const char * const roomjid,
-    const char * const message) {}
-void ui_room_member_offline(const char * const roomjid, const char * const nick) {}
-void ui_room_member_online(const char * const roomjid, const char * const nick, const char * const roles,
+void mucwin_broadcast(ProfMucWin *mucwin, const char * const message) {}
+void mucwin_occupant_offline(ProfMucWin *mucwin, const char * const nick) {}
+void mucwin_occupant_online(ProfMucWin *mucwin, const char * const nick, const char * const roles,
     const char * const affiliation, const char * const show, const char * const status) {}
-void ui_room_member_nick_change(const char * const roomjid,
-    const char * const old_nick, const char * const nick) {}
-void ui_room_nick_change(const char * const roomjid, const char * const nick) {}
-void ui_room_member_presence(const char * const roomjid,
-    const char * const nick, const char * const show, const char * const status) {}
-void ui_room_update_occupants(const char * const roomjid) {}
-void ui_room_show_occupants(const char * const roomjid) {}
-void ui_room_hide_occupants(const char * const roomjid) {}
+void mucwin_occupant_nick_change(ProfMucWin *mucwin, const char * const old_nick, const char * const nick) {}
+void mucwin_nick_change(ProfMucWin *mucwin, const char * const nick) {}
+void mucwin_occupant_presence(ProfMucWin *mucwin, const char * const nick, const char * const show,
+    const char * const status) {}
+void mucwin_update_occupants(ProfMucWin *mucwin) {}
+void mucwin_show_occupants(ProfMucWin *mucwin) {}
+void mucwin_hide_occupants(ProfMucWin *mucwin) {}
 void ui_show_roster(void) {}
 void ui_hide_roster(void) {}
 void ui_roster_add(const char * const barejid, const char * const name) {}
@@ -249,21 +231,11 @@ void ui_contact_already_in_group(const char * const contact, const char * const
 void ui_contact_not_in_group(const char * const contact, const char * const group) {}
 void ui_group_added(const char * const contact, const char * const group) {}
 void ui_group_removed(const char * const contact, const char * const group) {}
-void ui_chat_win_contact_online(PContact contact, Resource *resource, GDateTime *last_activity) {}
-void ui_chat_win_contact_offline(PContact contact, char *resource, char *status) {}
-gboolean ui_chat_win_exists(const char * const barejid)
-{
-    return TRUE;
-}
+void chatwin_contact_online(ProfChatWin *chatwin, Resource *resource, GDateTime *last_activity) {}
+void chatwin_contact_offline(ProfChatWin *chatwin, char *resource, char *status) {}
 
 void ui_contact_offline(char *barejid, char *resource, char *status) {}
 
-void ui_handle_recipient_not_found(const char * const recipient, const char * const err_msg)
-{
-    check_expected(recipient);
-    check_expected(err_msg);
-}
-
 void ui_handle_recipient_error(const char * const recipient, const char * const err_msg)
 {
     check_expected(recipient);
@@ -277,75 +249,65 @@ void ui_handle_error(const char * const err_msg)
 
 void ui_clear_win_title(void) {}
 void ui_goodbye_title(void) {}
-void ui_handle_room_join_error(const char * const roomjid, const char * const err) {}
-void ui_handle_room_configuration(const char * const roomjid, DataForm *form) {}
+void mucconfwin_handle_configuration(ProfMucConfWin *confwin, DataForm *form) {}
 void ui_handle_room_configuration_form_error(const char * const roomjid, const char * const message) {}
 void ui_handle_room_config_submit_result(const char * const roomjid) {}
 void ui_handle_room_config_submit_result_error(const char * const roomjid, const char * const message) {}
-void ui_handle_room_affiliation_list_error(const char * const roomjid, const char * const affiliation,
+void mucwin_affiliation_list_error(ProfMucWin *mucwin, const char * const affiliation, const char * const error) {}
+void mucwin_handle_affiliation_list(ProfMucWin *mucwin, const char * const affiliation, GSList *jids) {}
+void mucwin_affiliation_set_error(ProfMucWin *mucwin, const char * const jid, const char * const affiliation,
     const char * const error) {}
-void ui_handle_room_affiliation_list(const char * const roomjid, const char * const affiliation, GSList *jids) {}
-void ui_handle_room_affiliation_set_error(const char * const roomjid, const char * const jid,
-    const char * const affiliation, const char * const error) {}
-void ui_handle_room_role_set_error(const char * const roomjid, const char * const nick, const char * const role,
+void mucwin_role_set_error(ProfMucWin *mucwin, const char * const nick, const char * const role,
     const char * const error) {}
-void ui_handle_room_role_list_error(const char * const roomjid, const char * const role, const char * const error) {}
-void ui_handle_room_role_list(const char * const roomjid, const char * const role, GSList *nicks) {}
-void ui_handle_room_kick_error(const char * const roomjid, const char * const nick, const char * const error) {}
-void ui_show_form(ProfMucConfWin *confwin) {}
-void ui_show_form_field(ProfWin *window, DataForm *form, char *tag) {}
-void ui_show_form_help(ProfMucConfWin *confwin) {}
-void ui_show_form_field_help(ProfMucConfWin *confwin, char *tag) {}
-void ui_show_lines(ProfWin *window, const gchar** lines) {}
+void mucwin_role_list_error(ProfMucWin *mucwin, const char * const role, const char * const error) {}
+void mucwin_handle_role_list(ProfMucWin *mucwin, const char * const role, GSList *nicks) {}
+void mucwin_kick_error(ProfMucWin *mucwin, const char * const nick, const char * const error) {}
+void mucconfwin_show_form(ProfMucConfWin *confwin) {}
+void mucconfwin_show_form_field(ProfMucConfWin *confwin, DataForm *form, char *tag) {}
+void mucconfwin_form_help(ProfMucConfWin *confwin) {}
+void mucconfwin_field_help(ProfMucConfWin *confwin, char *tag) {}
+void ui_show_lines(ProfWin *window, gchar** lines) {}
 void ui_redraw_all_room_rosters(void) {}
 void ui_show_all_room_rosters(void) {}
 void ui_hide_all_room_rosters(void) {}
 
-gboolean ui_tidy_wins(void) {
+gboolean jabber_conn_is_secured(void)
+{
     return TRUE;
 }
-void ui_prune_wins(void) {}
-gboolean ui_swap_wins(int source_win, int target_win)
+TLSCertificate* jabber_get_tls_peer_cert(void)
 {
-    return FALSE;
+    return NULL;
 }
+void cons_show_tlscert(TLSCertificate *cert) {}
+void cons_show_tlscert_summary(TLSCertificate *cert) {}
 
-void ui_auto_away(void) {}
-void ui_end_auto_away(void) {}
-void ui_titlebar_presence(contact_presence_t presence) {}
-void ui_handle_login_account_success(ProfAccount *account) {}
+void ui_prune_wins(void) {}
+
+void ui_handle_login_account_success(ProfAccount *account, int secured) {}
 void ui_update_presence(const resource_presence_t resource_presence,
     const char * const message, const char * const show) {}
-void ui_about(void) {}
-void ui_statusbar_new(const int win) {}
 
-char*  ui_readline(void)
+char* inp_readline(void)
 {
     return NULL;
 }
 
+void inp_nonblocking(gboolean reset) {}
+
 void ui_inp_history_append(char *inp) {}
 
-void ui_input_clear(void) {}
-void ui_input_nonblocking(gboolean reset) {}
-
 void ui_invalid_command_usage(const char * const usage, void (*setting_func)(void)) {}
 
-void ui_create_xmlconsole_win(void) {}
-gboolean ui_xmlconsole_exists(void)
-{
-    return FALSE;
-}
-
-void ui_open_xmlconsole_win(void) {}
-
 gboolean ui_win_has_unsaved_form(int num)
 {
     return FALSE;
 }
 
-void
-ui_write(char *line, int offset) {}
+void ui_status_bar_inactive(const int win) {}
+void ui_status_bar_active(const int win) {}
+void ui_status_bar_new(const int win) {}
+void ui_write(char *line, int offset) {}
 
 // console window actions
 
@@ -360,7 +322,7 @@ void cons_show(const char * const msg, ...)
 
 void cons_show_padded(int pad, const char * const msg, ...) {}
 
-void cons_show_help(Command *command) {}
+void cons_show_help(const char *const cmd, CommandHelp *help) {}
 
 void cons_about(void) {}
 void cons_help(void) {}
@@ -407,18 +369,20 @@ cons_bad_cmd_usage(const char * const cmd)
 }
 
 void cons_show_roster_group(const char * const group, GSList * list) {}
-void cons_show_wins(void) {}
+void cons_show_wins(gboolean unread) {}
 void cons_show_status(const char * const barejid) {}
 void cons_show_info(PContact pcontact) {}
 void cons_show_caps(const char * const fulljid, resource_presence_t presence) {}
 void cons_show_themes(GSList *themes) {}
+void cons_show_scripts(GSList *scripts) {}
+void cons_show_script(const char *const script, GSList *commands) {}
 
 void cons_show_aliases(GList *aliases)
 {
     check_expected(aliases);
 }
 
-void cons_show_login_success(ProfAccount *account) {}
+void cons_show_login_success(ProfAccount *account, int secured) {}
 void cons_show_software_version(const char * const jid,
     const char * const presence, const char * const name,
     const char * const version, const char * const os) {}
@@ -441,7 +405,8 @@ void cons_show_room_invite(const char * const invitor, const char * const room,
     const char * const reason) {}
 void cons_check_version(gboolean not_available_msg) {}
 void cons_show_typing(const char * const barejid) {}
-void cons_show_incoming_message(const char * const short_from, const int win_index) {}
+void cons_show_incoming_room_message(const char *const nick, const char *const room, const int win_index, gboolean mention, GList *triggers, int unread) {}
+void cons_show_incoming_message(const char * const short_from, const int win_index, int unread) {}
 void cons_show_room_invites(GSList *invites) {}
 void cons_show_received_subs(void) {}
 void cons_show_sent_subs(void) {}
@@ -449,6 +414,7 @@ void cons_alert(void) {}
 void cons_theme_setting(void) {}
 void cons_privileges_setting(void) {}
 void cons_beep_setting(void) {}
+void cons_console_setting(void) {}
 void cons_flash_setting(void) {}
 void cons_splash_setting(void) {}
 void cons_vercheck_setting(void) {}
@@ -477,9 +443,9 @@ void cons_grlog_setting(void) {}
 void cons_autoaway_setting(void) {}
 void cons_reconnect_setting(void) {}
 void cons_autoping_setting(void) {}
-void cons_priority_setting(void) {}
 void cons_autoconnect_setting(void) {}
 void cons_inpblock_setting(void) {}
+void cons_tray_setting(void) {}
 
 void cons_show_contact_online(PContact contact, Resource *resource, GDateTime *last_activity)
 {
@@ -490,6 +456,10 @@ void cons_show_contact_online(PContact contact, Resource *resource, GDateTime *l
 
 void cons_show_contact_offline(PContact contact, char *resource, char *status) {}
 void cons_theme_colours(void) {}
+void cons_theme_properties(void) {}
+
+// title bar
+void title_bar_set_presence(contact_presence_t presence) {}
 
 // status bar
 void status_bar_inactive(const int win) {}
@@ -506,7 +476,7 @@ void occupantswin_occupants(const char * const room) {}
 // window interface
 ProfWin* win_create_console(void)
 {
-    return NULL;
+    return (ProfWin*)mock();
 }
 ProfWin* win_create_xmlconsole(void)
 {
@@ -528,9 +498,17 @@ ProfWin* win_create_private(const char * const fulljid)
 {
     return NULL;
 }
+ProfWin* win_create_plugin(const char *const plugin_name, const char * const tag)
+{
+    return NULL;
+}
 
 void win_update_virtual(ProfWin *window) {}
 void win_free(ProfWin *window) {}
+gboolean win_notify_remind(ProfWin *window)
+{
+    return TRUE;
+}
 int win_unread(ProfWin *window)
 {
     return 0;
@@ -552,15 +530,24 @@ void win_show_occupant_info(ProfWin *window, const char * const room, Occupant *
 void win_show_contact(ProfWin *window, PContact contact) {}
 void win_show_info(ProfWin *window, PContact contact) {}
 void win_println(ProfWin *window, int pad, const char * const message) {}
+void win_vprintln_ch(ProfWin *window, char ch, const char *const message, ...) {}
+void win_clear(ProfWin *window) {}
+char* win_get_string(ProfWin *window)
+{
+    return NULL;
+}
 
 // desktop notifier actions
 void notifier_uninit(void) {}
 
 void notify_typing(const char * const handle) {}
-void notify_message(ProfWin *window, const char * const name, const char * const text) {}
+void notify_message(const char *const name, int win, const char *const text) {}
 void notify_room_message(const char * const handle, const char * const room,
     int win, const char * const text) {}
 void notify_remind(void) {}
 void notify_invite(const char * const from, const char * const room,
     const char * const reason) {}
 void notify_subscription(const char * const from) {}
+void notify(const char * const message, int timeout,
+    const char * const category) {}
+
