@@ -8,8 +8,7 @@
 #include <sys/stat.h>
 
 #include "config.h"
-#include "chat_session.h"
-
+#include "xmpp/chat_session.h"
 #include "helpers.h"
 #include "test_autocomplete.h"
 #include "test_chat_session.h"
@@ -19,7 +18,7 @@
 #include "test_cmd_account.h"
 #include "test_cmd_rooms.h"
 #include "test_cmd_sub.h"
-#include "test_cmd_statuses.h"
+#include "test_cmd_presence.h"
 #include "test_cmd_otr.h"
 #include "test_cmd_pgp.h"
 #include "test_jid.h"
@@ -34,6 +33,8 @@
 #include "test_cmd_roster.h"
 #include "test_cmd_disconnect.h"
 #include "test_form.h"
+#include "test_callbacks.h"
+#include "test_plugins_disco.h"
 
 int main(int argc, char* argv[]) {
     const UnitTest all_tests[] = {
@@ -51,25 +52,6 @@ int main(int argc, char* argv[]) {
         unit_test(replace_when_sub_null),
         unit_test(replace_when_new_empty),
         unit_test(replace_when_new_null),
-        unit_test(compare_win_nums_less),
-        unit_test(compare_win_nums_equal),
-        unit_test(compare_win_nums_greater),
-        unit_test(compare_0s_equal),
-        unit_test(compare_0_greater_than_1),
-        unit_test(compare_1_less_than_0),
-        unit_test(compare_0_less_than_11),
-        unit_test(compare_11_greater_than_0),
-        unit_test(compare_0_greater_than_9),
-        unit_test(compare_9_less_than_0),
-        unit_test(next_available_when_only_console),
-        unit_test(next_available_3_at_end),
-        unit_test(next_available_9_at_end),
-        unit_test(next_available_0_at_end),
-        unit_test(next_available_2_in_first_gap),
-        unit_test(next_available_9_in_first_gap),
-        unit_test(next_available_0_in_first_gap),
-        unit_test(next_available_11_in_first_gap),
-        unit_test(next_available_24_first_big_gap),
         unit_test(test_online_is_valid_resource_presence_string),
         unit_test(test_chat_is_valid_resource_presence_string),
         unit_test(test_away_is_valid_resource_presence_string),
@@ -95,14 +77,6 @@ int main(int argc, char* argv[]) {
         unit_test(strip_quotes_strips_first),
         unit_test(strip_quotes_strips_last),
         unit_test(strip_quotes_strips_both),
-        unit_test(str_not_contains_str),
-        unit_test(str_contains_str_at_start),
-        unit_test(str_contains_str_at_end),
-        unit_test(str_contains_str_in_middle),
-        unit_test(str_contains_str_whole),
-        unit_test(str_empty_not_contains_str),
-        unit_test(str_not_contains_str_empty),
-        unit_test(str_empty_not_contains_str_empty),
 
         unit_test(clear_empty),
         unit_test(reset_after_create),
@@ -210,6 +184,20 @@ int main(int argc, char* argv[]) {
         unit_test(find_twice_returns_second_when_two_match),
         unit_test(find_five_times_finds_fifth),
         unit_test(find_twice_returns_first_when_two_match_and_reset),
+        unit_test(add_contact_with_no_group),
+        unit_test(add_contact_with_group),
+        unit_test(add_contact_with_two_groups),
+        unit_test(add_contact_with_three_groups),
+        unit_test(add_contact_with_three_groups_update_adding_two),
+        unit_test(add_contact_with_three_groups_update_removing_one),
+        unit_test(add_contact_with_three_groups_update_removing_two),
+        unit_test(add_contact_with_three_groups_update_removing_three),
+        unit_test(add_contact_with_three_groups_update_two_new),
+        unit_test(add_remove_contact_groups),
+        unit_test(add_contacts_with_different_groups),
+        unit_test(add_contacts_with_same_groups),
+        unit_test(add_contacts_with_overlapping_groups),
+        unit_test(remove_contact_with_remaining_in_group),
 
         unit_test_setup_teardown(returns_false_when_chat_session_does_not_exist,
             init_chat_sessions,
@@ -233,9 +221,6 @@ int main(int argc, char* argv[]) {
         unit_test_setup_teardown(cmd_connect_shows_message_when_connected,
             load_preferences,
             close_preferences),
-        unit_test_setup_teardown(cmd_connect_shows_message_when_undefined,
-            load_preferences,
-            close_preferences),
         unit_test_setup_teardown(cmd_connect_when_no_account,
             load_preferences,
             close_preferences),
@@ -303,8 +288,6 @@ int main(int argc, char* argv[]) {
         unit_test(cmd_rooms_shows_message_when_disconnected),
         unit_test(cmd_rooms_shows_message_when_disconnecting),
         unit_test(cmd_rooms_shows_message_when_connecting),
-        unit_test(cmd_rooms_shows_message_when_started),
-        unit_test(cmd_rooms_shows_message_when_undefined),
         unit_test(cmd_rooms_uses_account_default_when_no_arg),
         unit_test(cmd_rooms_arg_used_when_passed),
 
@@ -395,35 +378,35 @@ int main(int argc, char* argv[]) {
         unit_test(contact_available_when_highest_priority_online),
         unit_test(contact_available_when_highest_priority_chat),
 
-        unit_test(cmd_statuses_shows_usage_when_bad_subcmd),
-        unit_test(cmd_statuses_shows_usage_when_bad_console_setting),
-        unit_test(cmd_statuses_shows_usage_when_bad_chat_setting),
-        unit_test(cmd_statuses_shows_usage_when_bad_muc_setting),
-        unit_test_setup_teardown(cmd_statuses_console_sets_all,
+        unit_test(cmd_presence_shows_usage_when_bad_subcmd),
+        unit_test(cmd_presence_shows_usage_when_bad_console_setting),
+        unit_test(cmd_presence_shows_usage_when_bad_chat_setting),
+        unit_test(cmd_presence_shows_usage_when_bad_muc_setting),
+        unit_test_setup_teardown(cmd_presence_console_sets_all,
             load_preferences,
             close_preferences),
-        unit_test_setup_teardown(cmd_statuses_console_sets_online,
+        unit_test_setup_teardown(cmd_presence_console_sets_online,
             load_preferences,
             close_preferences),
-        unit_test_setup_teardown(cmd_statuses_console_sets_none,
+        unit_test_setup_teardown(cmd_presence_console_sets_none,
             load_preferences,
             close_preferences),
-        unit_test_setup_teardown(cmd_statuses_chat_sets_all,
+        unit_test_setup_teardown(cmd_presence_chat_sets_all,
             load_preferences,
             close_preferences),
-        unit_test_setup_teardown(cmd_statuses_chat_sets_online,
+        unit_test_setup_teardown(cmd_presence_chat_sets_online,
             load_preferences,
             close_preferences),
-        unit_test_setup_teardown(cmd_statuses_chat_sets_none,
+        unit_test_setup_teardown(cmd_presence_chat_sets_none,
             load_preferences,
             close_preferences),
-        unit_test_setup_teardown(cmd_statuses_muc_sets_all,
+        unit_test_setup_teardown(cmd_presence_room_sets_all,
             load_preferences,
             close_preferences),
-        unit_test_setup_teardown(cmd_statuses_muc_sets_online,
+        unit_test_setup_teardown(cmd_presence_room_sets_online,
             load_preferences,
             close_preferences),
-        unit_test_setup_teardown(cmd_statuses_muc_sets_none,
+        unit_test_setup_teardown(cmd_presence_room_sets_none,
             load_preferences,
             close_preferences),
 
@@ -479,8 +462,6 @@ int main(int argc, char* argv[]) {
         unit_test(cmd_bookmark_shows_message_when_disconnected),
         unit_test(cmd_bookmark_shows_message_when_disconnecting),
         unit_test(cmd_bookmark_shows_message_when_connecting),
-        unit_test(cmd_bookmark_shows_message_when_started),
-        unit_test(cmd_bookmark_shows_message_when_undefined),
         unit_test(cmd_bookmark_shows_usage_when_no_args),
         unit_test(cmd_bookmark_list_shows_bookmarks),
         unit_test(cmd_bookmark_add_shows_message_when_invalid_jid),
@@ -492,8 +473,6 @@ int main(int argc, char* argv[]) {
         unit_test(cmd_bookmark_remove_shows_message_when_no_bookmark),
 
 #ifdef HAVE_LIBOTR
-        unit_test(cmd_otr_shows_usage_when_no_args),
-        unit_test(cmd_otr_shows_usage_when_invalid_subcommand),
         unit_test(cmd_otr_log_shows_usage_when_no_args),
         unit_test(cmd_otr_log_shows_usage_when_invalid_subcommand),
         unit_test_setup_teardown(cmd_otr_log_on_enables_logging,
@@ -515,13 +494,9 @@ int main(int argc, char* argv[]) {
         unit_test(cmd_otr_gen_shows_message_when_not_connected),
         unit_test(cmd_otr_gen_generates_key_for_connected_account),
         unit_test(cmd_otr_gen_shows_message_when_disconnected),
-        unit_test(cmd_otr_gen_shows_message_when_undefined),
-        unit_test(cmd_otr_gen_shows_message_when_started),
         unit_test(cmd_otr_gen_shows_message_when_connecting),
         unit_test(cmd_otr_gen_shows_message_when_disconnecting),
         unit_test(cmd_otr_myfp_shows_message_when_disconnected),
-        unit_test(cmd_otr_myfp_shows_message_when_undefined),
-        unit_test(cmd_otr_myfp_shows_message_when_started),
         unit_test(cmd_otr_myfp_shows_message_when_connecting),
         unit_test(cmd_otr_myfp_shows_message_when_disconnecting),
         unit_test(cmd_otr_myfp_shows_message_when_no_key),
@@ -548,8 +523,6 @@ int main(int argc, char* argv[]) {
         unit_test(cmd_pgp_start_shows_message_when_disconnected),
         unit_test(cmd_pgp_start_shows_message_when_disconnecting),
         unit_test(cmd_pgp_start_shows_message_when_connecting),
-        unit_test(cmd_pgp_start_shows_message_when_undefined),
-        unit_test(cmd_pgp_start_shows_message_when_started),
         unit_test(cmd_pgp_start_shows_message_when_no_arg_in_console),
         unit_test(cmd_pgp_start_shows_message_when_no_arg_in_muc),
         unit_test(cmd_pgp_start_shows_message_when_no_arg_in_mucconf),
@@ -562,7 +535,6 @@ int main(int argc, char* argv[]) {
         unit_test(cmd_join_shows_message_when_disconnecting),
         unit_test(cmd_join_shows_message_when_connecting),
         unit_test(cmd_join_shows_message_when_disconnected),
-        unit_test(cmd_join_shows_message_when_undefined),
         unit_test(cmd_join_shows_error_message_when_invalid_room_jid),
         unit_test(cmd_join_uses_account_mucservice_when_no_service_specified),
         unit_test(cmd_join_uses_supplied_nick),
@@ -572,7 +544,6 @@ int main(int argc, char* argv[]) {
         unit_test(cmd_roster_shows_message_when_disconnecting),
         unit_test(cmd_roster_shows_message_when_connecting),
         unit_test(cmd_roster_shows_message_when_disconnected),
-        unit_test(cmd_roster_shows_message_when_undefined),
         unit_test(cmd_roster_shows_roster_when_no_args),
         unit_test(cmd_roster_add_shows_message_when_no_jid),
         unit_test(cmd_roster_add_sends_roster_add_request),
@@ -608,7 +579,23 @@ int main(int argc, char* argv[]) {
         unit_test(remove_text_multi_value_removes_when_one),
         unit_test(remove_text_multi_value_removes_when_many),
 
-        unit_test(clears_chat_sessions),
+        unit_test_setup_teardown(clears_chat_sessions,
+            load_preferences,
+            close_preferences),
+
+        unit_test(prof_partial_occurrences_tests),
+        unit_test(prof_whole_occurrences_tests),
+
+        unit_test(returns_no_commands),
+        unit_test(returns_commands),
+
+        unit_test(returns_empty_list_when_none),
+        unit_test(returns_added_feature),
+        unit_test(resets_features_on_close),
+        unit_test(returns_all_added_features),
+        unit_test(does_not_add_duplicate_feature),
+        unit_test(removes_plugin_features),
+        unit_test(does_not_remove_feature_when_more_than_one_reference),
     };
 
     return run_tests(all_tests);
